"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const dsUtils = require("../dataset/utils");
const vscode = require("vscode");
const fs = require("fs");
const globals = require("../globals");
const path = require("path");
const utils_1 = require("../utils");
const utils_2 = require("../shared/utils");
const Profiles_1 = require("../Profiles");
const ZoweExplorerApiRegister_1 = require("../api/ZoweExplorerApiRegister");
const imperative_1 = require("@zowe/imperative");
const icons_1 = require("../generators/icons");
const ZoweDatasetNode_1 = require("./ZoweDatasetNode");
const contextually = require("../shared/context");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Refreshes treeView
 *
 * @param {DataSetTree} datasetProvider
 */
function refreshAll(datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Profiles_1.Profiles.getInstance().refresh();
        datasetProvider.mSessionNodes.forEach((sessNode) => {
            if (contextually.isSessionNotFav(sessNode)) {
                utils_2.labelHack(sessNode);
                sessNode.children = [];
                sessNode.dirty = true;
                utils_2.refreshTree(sessNode);
            }
        });
        datasetProvider.refresh();
    });
}
exports.refreshAll = refreshAll;
function uploadDialog(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileOpenOptions = {
            canSelectFiles: true,
            openLabel: "Upload File",
            canSelectMany: true
        };
        const value = yield vscode.window.showOpenDialog(fileOpenOptions);
        if (value && value.length) {
            yield Promise.all(value.map((item) => __awaiter(this, void 0, void 0, function* () {
                // Convert to vscode.TextDocument
                const doc = yield vscode.workspace.openTextDocument(item);
                yield uploadFile(node, doc);
            })));
            // refresh Tree View & favorites
            datasetProvider.refreshElement(node);
            if (contextually.isFavorite(node) || contextually.isFavoriteContext(node.getParent())) {
                const nonFavNode = datasetProvider.findNonFavoritedNode(node);
                if (nonFavNode) {
                    datasetProvider.refreshElement(nonFavNode);
                }
            }
            else {
                const favNode = datasetProvider.findFavoritedNode(node);
                if (favNode) {
                    datasetProvider.refreshElement(favNode);
                }
            }
        }
        else {
            vscode.window.showInformationMessage(localize(0, null));
        }
    });
}
exports.uploadDialog = uploadDialog;
function uploadFile(node, doc) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const datasetName = dsUtils.getDatasetLabel(node);
            yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).putContents(doc.fileName, datasetName);
        }
        catch (e) {
            utils_1.errorHandling(e, node.getProfileName(), e.message);
        }
    });
}
exports.uploadFile = uploadFile;
/**
 * Creates a PDS member
 *
 * @export
 * @param {IZoweDatasetTreeNode} parent - The parent Node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function createMember(parent, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const name = yield vscode.window.showInputBox({ placeHolder: localize(1, null) });
        globals.LOG.debug(localize(2, null) + name);
        if (name) {
            let label = parent.label.trim();
            if (contextually.isFavoritePds(parent)) {
                label = parent.label.substring(parent.label.indexOf(":") + 2); // TODO MISSED TESTING
            }
            try {
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(parent.getProfile()).createDataSetMember(label + "(" + name + ")");
            }
            catch (err) {
                globals.LOG.error(localize(3, null) + JSON.stringify(err));
                utils_1.errorHandling(err, label, localize(4, null) + err.message);
                throw (err);
            }
            parent.dirty = true;
            datasetProvider.refreshElement(parent);
            openPS(new ZoweDatasetNode_1.ZoweDatasetNode(name, vscode.TreeItemCollapsibleState.None, parent, null, undefined, undefined, parent.getProfile()), true, datasetProvider);
            datasetProvider.refresh();
        }
    });
}
exports.createMember = createMember;
/**
 * Downloads and displays a PS in a text editor view
 *
 * @param {IZoweDatasetTreeNode} node
 */
function openPS(node, previewMember, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        // let sesNamePrompt: string;
        // if (node.contextValue.endsWith(globals.FAV_SUFFIX)) {
        //     sesNamePrompt = node.getLabel().substring(1, node.getLabel().indexOf("]"));
        // } else {
        //     sesNamePrompt = node.getLabel();
        // }
        if (datasetProvider) {
            yield datasetProvider.checkCurrentProfile(node);
        }
        if (Profiles_1.Profiles.getInstance().validProfile === Profiles_1.ValidProfileEnum.VALID) {
            try {
                let label;
                switch (true) {
                    case contextually.isFavoriteContext(node.getParent()):
                        label = node.label.substring(node.label.indexOf(":") + 1).trim();
                        break;
                    case contextually.isFavoritePds(node.getParent()):
                        label = node.getParent().getLabel().substring(node.getParent().getLabel().indexOf(":") + 1).trim() + "(" + node.getLabel() + ")";
                        break;
                    case contextually.isSessionNotFav(node.getParent()):
                        label = node.label.trim();
                        break;
                    case contextually.isPdsNotFav(node.getParent()):
                        label = node.getParent().getLabel().trim() + "(" + node.getLabel() + ")";
                        break;
                    default:
                        vscode.window.showErrorMessage(localize(5, null));
                        throw Error(localize(6, null));
                }
                globals.LOG.debug(localize(7, null) + label);
                // if local copy exists, open that instead of pulling from mainframe
                const documentFilePath = utils_2.getDocumentFilePath(label, node);
                if (!fs.existsSync(documentFilePath)) {
                    const response = yield vscode.window.withProgress({
                        location: vscode.ProgressLocation.Notification,
                        title: "Opening data set..."
                    }, function downloadDataset() {
                        return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).getContents(label, {
                            file: documentFilePath,
                            returnEtag: true
                        });
                    });
                    node.setEtag(response.apiResponse.etag);
                }
                const document = yield vscode.workspace.openTextDocument(utils_2.getDocumentFilePath(label, node));
                if (previewMember === true) {
                    yield vscode.window.showTextDocument(document);
                }
                else {
                    yield vscode.window.showTextDocument(document, { preview: false });
                }
            }
            catch (err) {
                globals.LOG.error(localize(8, null) + JSON.stringify(err));
                utils_1.errorHandling(err, node.getProfileName(), err.message);
                throw (err);
            }
        }
    });
}
exports.openPS = openPS;
/**
 * Creates a new file and uploads to the server
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * TODO: Consider changing configuration to allow "custom" data set specifications
 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 * @export
 * @param {IZoweDatasetTreeNode} node - Desired Zowe session
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function createFile(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const quickPickOptions = {
            placeHolder: localize(9, null),
            ignoreFocusOut: true,
            canPickMany: false
        };
        const types = [
            localize(10, null),
            localize(11, null),
            localize(12, null),
            localize(13, null),
            localize(14, null)
        ];
        // let sesNamePrompt: string;
        // if (node.contextValue.endsWith(globals.FAV_SUFFIX)) {
        //     sesNamePrompt = node.label.substring(1, node.label.indexOf("]"));
        // } else {
        //     sesNamePrompt = node.label;
        // }
        datasetProvider.checkCurrentProfile(node);
        if (Profiles_1.Profiles.getInstance().validProfile === Profiles_1.ValidProfileEnum.VALID) {
            // get data set type
            const type = yield vscode.window.showQuickPick(types, quickPickOptions);
            if (type == null) {
                globals.LOG.debug(localize(15, null));
                return;
            }
            else {
                globals.LOG.debug(localize(16, null));
            }
            let typeEnum;
            let createOptions;
            switch (type) {
                case localize(17, null):
                    typeEnum = 0 /* DATA_SET_BINARY */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-Binary");
                    break;
                case localize(18, null):
                    typeEnum = 1 /* DATA_SET_C */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-C");
                    break;
                case localize(19, null):
                    typeEnum = 2 /* DATA_SET_CLASSIC */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-Classic");
                    break;
                case localize(20, null):
                    typeEnum = 3 /* DATA_SET_PARTITIONED */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-PDS");
                    break;
                case localize(21, null):
                    typeEnum = 4 /* DATA_SET_SEQUENTIAL */;
                    createOptions = vscode.workspace.getConfiguration("Zowe-Default-Datasets-PS");
                    break;
            }
            // get name of data set
            let name = yield vscode.window.showInputBox({ placeHolder: localize(22, null) });
            name = name.toUpperCase();
            try {
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).createDataSet(typeEnum, name, createOptions);
                node.dirty = true;
                const theFilter = yield datasetProvider.createFilterString(name, node);
                datasetProvider.addHistory(theFilter);
                datasetProvider.refresh();
                // Show newly-created data set in expanded tree view
                if (name) {
                    node.label = `${node.label} `;
                    node.label = node.label.trim();
                    node.tooltip = node.pattern = theFilter.toUpperCase();
                    node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                    const icon = icons_1.getIconByNode(node);
                    if (icon) {
                        node.iconPath = icon.path;
                    }
                    node.dirty = true;
                    const newNode = yield node.getChildren().then((children) => children.find((child) => child.label === name));
                    datasetProvider.getTreeView().reveal(newNode, { select: true });
                }
            }
            catch (err) {
                globals.LOG.error(localize(23, null) + JSON.stringify(err));
                utils_1.errorHandling(err, node.getProfileName(), localize(24, null) +
                    err.message);
                throw (err);
            }
        }
    });
}
exports.createFile = createFile;
/**
 * Shows data set attributes in a new text editor
 *
 * @export
 * @param {IZoweDatasetTreeNode} parent - The parent Node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function showDSAttributes(parent, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let label = parent.label.trim();
        if (contextually.isFavoritePds(parent) || contextually.isFavoriteDs(parent)) {
            label = parent.label.trim().substring(parent.label.trim().indexOf(":") + 2);
        }
        globals.LOG.debug(localize(25, null) + label);
        let attributes;
        try {
            attributes = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(parent.getProfile()).dataSet(label, { attributes: true });
            attributes = attributes.apiResponse.items;
            attributes = attributes.filter((dataSet) => {
                return dataSet.dsname.toUpperCase() === label.toUpperCase();
            });
            if (attributes.length === 0) {
                throw new Error(localize(26, null) + label);
            }
        }
        catch (err) {
            globals.LOG.error(localize(27, null) + JSON.stringify(err));
            utils_1.errorHandling(err, parent.getProfileName(), localize(28, null) + err.message);
            throw (err);
        }
        // shouldn't be possible for there to be two cataloged data sets with the same name,
        // but just in case we'll display all of the results
        // if there's only one result (which there should be), we will just pass in attributes[0]
        // so that prettyJson doesn't display the attributes as an array with a hyphen character
        const attributesText = imperative_1.TextUtils.prettyJson(attributes.length > 1 ? attributes : attributes[0], undefined, false);
        // const attributesFilePath = path.join(ZOWETEMPFOLDER, label + ".yaml");
        // fs.writeFileSync(attributesFilePath, attributesText);
        // const document = await vscode.workspace.openTextDocument(attributesFilePath);
        // await vscode.window.showTextDocument(document);
        const attributesMessage = localize(29, null);
        const webviewHTML = `<!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <title>${label} "${attributesMessage}"</title>
    </head>
    <body>
     ${attributesText.replace(/\n/g, "</br>")}
    </body>
    </html>`;
        const column = vscode.window.activeTextEditor
            ? vscode.window.activeTextEditor.viewColumn
            : undefined;
        const panel = vscode.window.createWebviewPanel("zowe", label + " " + localize(30, null), column || 1, {});
        panel.webview.html = webviewHTML;
    });
}
exports.showDSAttributes = showDSAttributes;
/**
 * Rename data sets
 *
 * @export
 * @param {IZoweDatasetTreeNode} node - The node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function renameDataSet(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let beforeDataSetName = node.label.trim();
        let favPrefix = "";
        let isFavourite;
        if (node.contextValue.includes(globals.FAV_SUFFIX)) {
            isFavourite = true;
            favPrefix = node.label.substring(0, node.label.indexOf(":") + 2);
            beforeDataSetName = node.label.substring(node.label.indexOf(":") + 2);
        }
        const afterDataSetName = yield vscode.window.showInputBox({ value: beforeDataSetName });
        const beforeFullPath = utils_2.getDocumentFilePath(node.getLabel(), node);
        const closedOpenedInstance = yield closeOpenedTextFile(beforeFullPath);
        globals.LOG.debug(localize(31, null) + afterDataSetName);
        if (afterDataSetName) {
            try {
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).renameDataSet(beforeDataSetName, afterDataSetName);
                node.label = `${favPrefix}${afterDataSetName}`;
                if (isFavourite) {
                    const profile = favPrefix.substring(1, favPrefix.indexOf("]"));
                    datasetProvider.renameNode(profile, beforeDataSetName, afterDataSetName);
                }
                else {
                    const temp = node.label;
                    node.label = "[" + node.getSessionNode().label.trim() + "]: " + beforeDataSetName;
                    datasetProvider.renameFavorite(node, afterDataSetName);
                    node.label = temp;
                }
                datasetProvider.refreshElement(node);
                datasetProvider.updateFavorites();
                if (fs.existsSync(beforeFullPath)) {
                    fs.unlinkSync(beforeFullPath);
                }
                if (closedOpenedInstance) {
                    vscode.commands.executeCommand("zowe.ZoweNode.openPS", node);
                }
            }
            catch (err) {
                globals.LOG.error(localize(32, null) + JSON.stringify(err));
                utils_1.errorHandling(err, favPrefix, localize(33, null) + err.message);
                throw err;
            }
        }
    });
}
exports.renameDataSet = renameDataSet;
/**
 * Submit the contents of the editor as JCL.
 *
 * @export
 * @param {DatasetTree} datasetProvider - our DatasetTree object
 */
function submitJcl(datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!vscode.window.activeTextEditor) {
            vscode.window.showErrorMessage(localize(34, null));
            return;
        }
        const doc = vscode.window.activeTextEditor.document;
        globals.LOG.debug(localize(35, null) + doc.fileName);
        // get session name
        const sessionregex = /\[(.*)(\])(?!.*\])/g;
        const regExp = sessionregex.exec(doc.fileName);
        const profiles = Profiles_1.Profiles.getInstance();
        let sessProfileName;
        if (regExp === null) {
            const allProfiles = profiles.allProfiles;
            const profileNamesList = allProfiles.map((profile) => {
                return profile.name;
            });
            if (profileNamesList.length) {
                const quickPickOptions = {
                    placeHolder: localize(36, null),
                    ignoreFocusOut: true,
                    canPickMany: false
                };
                sessProfileName = yield vscode.window.showQuickPick(profileNamesList, quickPickOptions);
            }
            else {
                vscode.window.showInformationMessage(localize(37, null));
            }
        }
        else {
            sessProfileName = regExp[1];
            if (sessProfileName.includes("[")) {
                // if submitting from favorites, sesName might be the favorite node, so extract further
                sessProfileName = sessionregex.exec(sessProfileName)[1];
            }
        }
        // get profile from session name
        let sessProfile;
        const sesNode = (yield datasetProvider.getChildren()).find((child) => child.label.trim() === sessProfileName);
        if (sesNode) {
            sessProfile = sesNode.getProfile();
        }
        else {
            // if submitting from favorites, a session might not exist for this node
            sessProfile = profiles.loadNamedProfile(sessProfileName);
        }
        if (sessProfile == null) {
            globals.LOG.error(localize(38, null));
            return;
        }
        try {
            const job = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(sessProfile).submitJcl(doc.getText());
            const args = [sessProfileName, job.jobid];
            const setJobCmd = `command:zowe.setJobSpool?${encodeURIComponent(JSON.stringify(args))}`;
            vscode.window.showInformationMessage(localize(39, null) + `[${job.jobid}](${setJobCmd})`);
        }
        catch (error) {
            utils_1.errorHandling(error, sessProfileName, localize(40, null) + error.message);
        }
    });
}
exports.submitJcl = submitJcl;
/**
 * Submit the selected dataset member as a Job.
 *
 * @export
 * @param node The dataset member
 */
function submitMember(node) {
    return __awaiter(this, void 0, void 0, function* () {
        const labelregex = /\[(.+)\]\: (.+)/g;
        let label;
        let sesName;
        let sessProfile;
        let regex;
        const profiles = Profiles_1.Profiles.getInstance();
        switch (true) {
            case contextually.isFavoriteContext(node.getParent()):
                regex = labelregex.exec(node.getLabel());
                sesName = regex[1];
                label = regex[2];
                sessProfile = profiles.loadNamedProfile(sesName);
                break;
            case contextually.isFavoritePds(node.getParent()):
                regex = labelregex.exec(node.getParent().getLabel());
                sesName = regex[1];
                label = regex[2] + "(" + node.label.trim() + ")";
                sessProfile = node.getParent().getProfile();
                break;
            case contextually.isSessionNotFav(node.getParent()):
                sesName = node.getParent().getLabel();
                label = node.label;
                sessProfile = node.getParent().getProfile();
                break;
            case contextually.isPdsNotFav(node.getParent()):
                sesName = node.getParent().getParent().getLabel();
                label = node.getParent().getLabel() + "(" + node.label.trim() + ")";
                sessProfile = node.getParent().getParent().getProfile();
                break;
            default:
                vscode.window.showErrorMessage(localize(41, null));
                throw Error(localize(42, null));
        }
        try {
            const job = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getJesApi(sessProfile).submitJob(label);
            const args = [sesName, job.jobid];
            const setJobCmd = `command:zowe.setJobSpool?${encodeURIComponent(JSON.stringify(args))}`;
            vscode.window.showInformationMessage(localize(43, null) + `[${job.jobid}](${setJobCmd})`);
        }
        catch (error) {
            utils_1.errorHandling(error, sesName, localize(44, null) + error.message);
        }
    });
}
exports.submitMember = submitMember;
/**
 * Rename data set members
 *
 * @export
 * @param {IZoweTreeNode} node - The node
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function renameDataSetMember(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const beforeMemberName = node.label.trim();
        let dataSetName;
        let profileLabel;
        if (node.getParent().contextValue.includes(globals.FAV_SUFFIX)) {
            profileLabel = node.getParent().getLabel().substring(0, node.getParent().getLabel().indexOf(":") + 2);
            dataSetName = node.getParent().getLabel().substring(node.getParent().getLabel().indexOf(":") + 2);
        }
        else {
            dataSetName = node.getParent().getLabel();
        }
        const afterMemberName = yield vscode.window.showInputBox({ value: beforeMemberName });
        const beforeFullPath = utils_2.getDocumentFilePath(`${node.getParent().getLabel()}(${node.getLabel()})`, node);
        const closedOpenedInstance = yield closeOpenedTextFile(beforeFullPath);
        globals.LOG.debug(localize(45, null) + afterMemberName);
        if (afterMemberName) {
            try {
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).renameDataSetMember(dataSetName, beforeMemberName, afterMemberName);
                node.label = afterMemberName;
            }
            catch (err) {
                globals.LOG.error(localize(46, null) + JSON.stringify(err));
                utils_1.errorHandling(err, profileLabel, localize(47, null) + err.message);
                throw err;
            }
            let otherParent;
            if (node.getParent().contextValue.includes(globals.FAV_SUFFIX)) {
                otherParent = datasetProvider.findNonFavoritedNode(node.getParent());
            }
            else {
                otherParent = datasetProvider.findFavoritedNode(node.getParent());
            }
            if (otherParent) {
                const otherMember = otherParent.children.find((child) => child.label === beforeMemberName);
                if (otherMember) {
                    otherMember.label = afterMemberName;
                    datasetProvider.refreshElement(otherMember);
                }
            }
            datasetProvider.refreshElement(node);
            if (fs.existsSync(beforeFullPath)) {
                fs.unlinkSync(beforeFullPath);
            }
            if (closedOpenedInstance) {
                vscode.commands.executeCommand("zowe.ZoweNode.openPS", node);
            }
        }
    });
}
exports.renameDataSetMember = renameDataSetMember;
/**
 * Deletes a dataset
 *
 * @export
 * @param {IZoweTreeNode} node - The node to be deleted
 * @param {IZoweTree<IZoweDatasetTreeNode>} datasetProvider - the tree which contains the nodes
 */
function deleteDataset(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        globals.LOG.debug(localize(48, null) + node.label);
        const quickPickOptions = {
            placeHolder: localize(49, null) + node.label,
            ignoreFocusOut: true,
            canPickMany: false
        };
        // confirm that the user really wants to delete
        if ((yield vscode.window.showQuickPick([localize(50, null),
            localize(51, null)], quickPickOptions)) !== localize(52, null)) {
            globals.LOG.debug(localize(53, null));
            return;
        }
        let label = "";
        let fav = false;
        try {
            switch (node.getParent().contextValue) {
                case (globals.FAVORITE_CONTEXT):
                    label = node.label.substring(node.label.indexOf(":") + 1).trim();
                    fav = true;
                    break;
                case (globals.DS_PDS_CONTEXT + globals.FAV_SUFFIX):
                    label = node.getParent().getLabel().substring(node.getParent().getLabel().indexOf(":") + 1).trim() + "(" + node.getLabel() + ")";
                    fav = true;
                    break;
                case (globals.DS_SESSION_CONTEXT):
                    label = node.getLabel();
                    break;
                case (globals.DS_PDS_CONTEXT):
                    label = node.getParent().getLabel() + "(" + node.getLabel() + ")";
                    break;
                default:
                    throw Error(localize(54, null));
            }
            yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).deleteDataSet(label);
        }
        catch (err) {
            globals.LOG.error(localize(55, null) + JSON.stringify(err));
            if (err.message.includes(localize(56, null))) {
                vscode.window.showInformationMessage(localize(57, null) + label +
                    localize(58, null));
            }
            else {
                utils_1.errorHandling(err, node.getProfileName(), err.message);
            }
            throw err;
        }
        // remove node from tree
        if (fav) {
            datasetProvider.mSessionNodes.forEach((ses) => {
                if (node.label.substring(node.label.indexOf("[") + 1, node.label.indexOf("]")) === ses.label.trim() ||
                    node.getParent().getLabel().substring(node.getParent().getLabel().indexOf("["), node.getParent().getLabel().indexOf("]")) === ses.label) {
                    ses.dirty = true;
                }
            });
            datasetProvider.removeFavorite(node);
        }
        else {
            node.getSessionNode().dirty = true;
            const temp = node.label;
            node.label = "[" + node.getSessionNode().label.trim() + "]: " + node.label;
            datasetProvider.removeFavorite(node);
            node.label = temp;
        }
        // refresh Tree View & favorites
        if (node.getParent() && node.getParent().contextValue !== globals.DS_SESSION_CONTEXT) {
            datasetProvider.refreshElement(node.getParent());
            if (contextually.isFavorite(node) || contextually.isFavoriteContext(node.getParent())) {
                const nonFavNode = datasetProvider.findNonFavoritedNode(node.getParent());
                if (nonFavNode) {
                    datasetProvider.refreshElement(nonFavNode);
                }
            }
            else {
                const favNode = datasetProvider.findFavoritedNode(node.getParent());
                if (favNode) {
                    datasetProvider.refreshElement(favNode);
                }
            }
        }
        else {
            datasetProvider.refresh();
        }
        // remove local copy of file
        const fileName = utils_2.getDocumentFilePath(label, node);
        try {
            if (fs.existsSync(fileName)) {
                fs.unlinkSync(fileName);
            }
        }
        catch (err) {
            // do nothing
        }
    });
}
exports.deleteDataset = deleteDataset;
/**
 * Refreshes the passed node with current mainframe data
 *
 * @param {IZoweDatasetTreeNode} node - The node which represents the dataset
 */
function refreshPS(node) {
    return __awaiter(this, void 0, void 0, function* () {
        let label;
        try {
            switch (true) {
                case contextually.isFavoriteContext(node.getParent()):
                    label = node.label.substring(node.label.indexOf(":") + 1).trim();
                    break;
                case contextually.isFavoritePds(node.getParent()):
                    label = node.getParent().getLabel().substring(node.getParent().getLabel().indexOf(":") + 1).trim() + "(" + node.getLabel() + ")";
                    break;
                case contextually.isSessionNotFav(node.getParent()):
                    label = node.label.trim();
                    break;
                case contextually.isPdsNotFav(node.getParent()):
                    label = node.getParent().getLabel() + "(" + node.getLabel() + ")";
                    break;
                default:
                    throw Error(localize(59, null));
            }
            const documentFilePath = utils_2.getDocumentFilePath(label, node);
            const response = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).getContents(label, {
                file: documentFilePath,
                returnEtag: true
            });
            node.setEtag(response.apiResponse.etag);
            const document = yield vscode.workspace.openTextDocument(documentFilePath);
            vscode.window.showTextDocument(document);
            // if there are unsaved changes, vscode won't automatically display the updates, so close and reopen
            if (document.isDirty) {
                yield vscode.commands.executeCommand("workbench.action.closeActiveEditor");
                vscode.window.showTextDocument(document);
            }
        }
        catch (err) {
            globals.LOG.error(localize(60, null) + JSON.stringify(err));
            if (err.message.includes(localize(61, null))) {
                vscode.window.showInformationMessage(localize(62, null) + label +
                    localize(63, null));
            }
            else {
                utils_1.errorHandling(err, node.getProfileName(), err.message);
            }
        }
    });
}
exports.refreshPS = refreshPS;
/**
 * Prompts the user for a pattern, and populates the [TreeView]{@link vscode.TreeView} based on the pattern
 *
 * @param {IZoweDatasetTreeNode} node - The session node
 * @param {DatasetTree} datasetProvider - Current DatasetTree used to populate the TreeView
 * @returns {Promise<void>}
 */
function enterPattern(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        if (globals.LOG) {
            globals.LOG.debug(localize(64, null));
        }
        let pattern;
        if (contextually.isSessionNotFav(node)) {
            // manually entering a search
            const options = {
                prompt: localize(65, null),
                value: node.pattern
            };
            // get user input
            pattern = yield vscode.window.showInputBox(options);
            if (!pattern) {
                vscode.window.showInformationMessage(localize(66, null));
                return;
            }
        }
        else {
            // executing search from saved search in favorites
            pattern = node.label.trim().substring(node.label.trim().indexOf(":") + 2);
            const session = node.label.trim().substring(node.label.trim().indexOf("[") + 1, node.label.trim().indexOf("]"));
            yield datasetProvider.addSession(session);
            node = datasetProvider.mSessionNodes.find((tempNode) => tempNode.label.trim() === session);
        }
        // update the treeview with the new pattern
        // TODO figure out why a label change is needed to refresh the treeview,
        // instead of changing the collapsible state
        // change label so the treeview updates
        node.label = node.label.trim() + " ";
        node.label = node.label.trim();
        node.tooltip = node.pattern = pattern.toUpperCase();
        node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
        node.dirty = true;
        const icon = icons_1.getIconByNode(node);
        if (icon) {
            node.iconPath = icon.path;
        }
        datasetProvider.addHistory(node.pattern);
    });
}
exports.enterPattern = enterPattern;
/**
 * Copy data sets
 *
 * @export
 * @param {IZoweNodeType} node - The node to copy
 */
function copyDataSet(node) {
    return __awaiter(this, void 0, void 0, function* () {
        return vscode.env.clipboard.writeText(JSON.stringify(dsUtils.getNodeLabels(node)));
    });
}
exports.copyDataSet = copyDataSet;
/**
 * Migrate data sets
 *
 * @export
 * @param {IZoweDatasetTreeNode} node - The node to paste to
 */
function hMigrateDataSet(node) {
    return __awaiter(this, void 0, void 0, function* () {
        const { dataSetName } = dsUtils.getNodeLabels(node);
        vscode.window.showInformationMessage(localize(67, null) + dataSetName +
            localize(68, null));
        return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).hMigrateDataSet(dataSetName);
    });
}
exports.hMigrateDataSet = hMigrateDataSet;
/**
 * Paste data sets
 *
 * @export
 * @param {ZoweNode} node - The node to paste to
 * @param {DatasetTree} datasetProvider - the tree which contains the nodes
 */
function pasteDataSet(node, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const { profileName, dataSetName } = dsUtils.getNodeLabels(node);
        let memberName;
        let beforeDataSetName;
        let beforeProfileName;
        let beforeMemberName;
        if (node.contextValue.includes(globals.DS_PDS_CONTEXT)) {
            memberName = yield vscode.window.showInputBox({ placeHolder: localize(69, null) });
            if (!memberName) {
                return;
            }
        }
        try {
            ({
                dataSetName: beforeDataSetName,
                memberName: beforeMemberName,
                profileName: beforeProfileName,
            } = JSON.parse(yield vscode.env.clipboard.readText()));
        }
        catch (err) {
            throw Error("Invalid clipboard. Copy from data set first");
        }
        if (beforeProfileName === profileName) {
            if (memberName) {
                const responseItem = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).allMembers(`${dataSetName}`);
                if (responseItem.apiResponse.items.some((singleItem) => singleItem.member === memberName.toUpperCase())) {
                    throw Error(`${dataSetName}(${memberName}) already exists. You cannot replace a member`);
                }
            }
            yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node.getProfile()).copyDataSetMember({ dataSetName: beforeDataSetName, memberName: beforeMemberName }, { dataSetName, memberName });
            if (memberName) {
                datasetProvider.refreshElement(node);
                let node2;
                if (node.contextValue.includes(globals.FAV_SUFFIX)) {
                    node2 = datasetProvider.findNonFavoritedNode(node);
                }
                else {
                    node2 = datasetProvider.findFavoritedNode(node);
                }
                if (node2) {
                    datasetProvider.refreshElement(node2);
                }
            }
            else {
                refreshPS(node);
            }
        }
    });
}
exports.pasteDataSet = pasteDataSet;
/**
 * Uploads the file to the mainframe
 *
 * @export
 * @param {vscode.TextDocument} doc - TextDocument that is being saved
 */
function saveFile(doc, datasetProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        // Check if file is a data set, instead of some other file
        globals.LOG.debug(localize(70, null) + doc.fileName);
        const docPath = path.join(doc.fileName, "..");
        globals.LOG.debug("requested to save data set: " + doc.fileName);
        if (docPath.toUpperCase().indexOf(globals.DS_DIR.toUpperCase()) === -1) {
            globals.LOG.debug(localize(71, null) +
                localize(72, null) + path.relative(docPath, globals.DS_DIR));
            return;
        }
        const start = path.join(globals.DS_DIR + path.sep).length;
        const ending = doc.fileName.substring(start);
        const sesName = ending.substring(0, ending.indexOf(path.sep));
        const profile = (yield Profiles_1.Profiles.getInstance()).loadNamedProfile(sesName);
        if (!profile) {
            globals.LOG.error(localize(73, null));
            return vscode.window.showErrorMessage(localize(74, null));
        }
        // get session from session name
        let documentSession;
        let node;
        const sesNode = (yield datasetProvider.getChildren()).find((child) => child.label.trim() === sesName);
        if (sesNode) {
            globals.LOG.debug(localize(75, null));
            documentSession = sesNode.getSession();
        }
        else {
            // if saving from favorites, a session might not exist for this node
            globals.LOG.debug(localize(76, null));
            documentSession = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).getSession();
        }
        // If not a member
        let label = doc.fileName.substring(doc.fileName.lastIndexOf(path.sep) + 1, utils_2.checkForAddedSuffix(doc.fileName) ? doc.fileName.lastIndexOf(".") : doc.fileName.length);
        label = label.toUpperCase();
        globals.LOG.debug(localize(77, null) + label);
        if (!label.includes("(")) {
            try {
                // Checks if file still exists on server
                const response = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(profile).dataSet(label);
                if (!response.apiResponse.items.length) {
                    return vscode.window.showErrorMessage(localize(78, null));
                }
            }
            catch (err) {
                utils_1.errorHandling(err, sesName, err.message);
            }
        }
        // Get specific node based on label and parent tree (session / favorites)
        let nodes;
        let isFromFavorites;
        if (!sesNode || sesNode.children.length === 0) {
            // saving from favorites
            nodes = utils_2.concatChildNodes(datasetProvider.mFavorites);
            isFromFavorites = true;
        }
        else {
            // saving from session
            nodes = utils_2.concatChildNodes([sesNode]);
            isFromFavorites = false;
        }
        node = nodes.find((zNode) => {
            // dataset in Favorites
            if (contextually.isFavoriteDs(zNode)) {
                return (zNode.label === `[${sesName}]: ${label}`);
                // member in Favorites
            }
            else if (contextually.isDsMember(zNode) && isFromFavorites) {
                const zNodeDetails = dsUtils.getProfileAndDataSetName(zNode);
                return (`${zNodeDetails.profileName}(${zNodeDetails.dataSetName})` === `[${sesName}]: ${label}`);
            }
            else if (contextually.isDsMember(zNode) && !isFromFavorites) {
                const zNodeDetails = dsUtils.getProfileAndDataSetName(zNode);
                return (`${zNodeDetails.profileName}(${zNodeDetails.dataSetName})` === `${label}`);
            }
            else if (contextually.isDs(zNode)) {
                return (zNode.label.trim() === label);
            }
            else {
                return false;
            }
        });
        // define upload options
        let uploadOptions;
        if (node) {
            uploadOptions = {
                etag: node.getEtag(),
                returnEtag: true
            };
        }
        try {
            const uploadResponse = yield vscode.window.withProgress({
                location: vscode.ProgressLocation.Notification,
                title: localize(79, null)
            }, () => {
                return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node ? node.getProfile() : profile).putContents(doc.fileName, label, uploadOptions);
            });
            if (uploadResponse.success) {
                vscode.window.showInformationMessage(uploadResponse.commandResponse);
                // set local etag with the new etag from the updated file on mainframe
                if (node) {
                    node.setEtag(uploadResponse.apiResponse[0].etag);
                }
            }
            else if (!uploadResponse.success && uploadResponse.commandResponse.includes(localize(80, null))) {
                const downloadResponse = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getMvsApi(node ? node.getProfile() : profile).getContents(label, {
                    file: doc.fileName,
                    returnEtag: true
                });
                // re-assign etag, so that it can be used with subsequent requests
                const downloadEtag = downloadResponse.apiResponse.etag;
                if (node && downloadEtag !== node.getEtag()) {
                    node.setEtag(downloadEtag);
                }
                vscode.window.showWarningMessage(localize(81, null));
                // Store document in a separate variable, to be used on merge conflict
                const oldDoc = doc;
                const oldDocText = oldDoc.getText();
                const startPosition = new vscode.Position(0, 0);
                const endPosition = new vscode.Position(oldDoc.lineCount, 0);
                const deleteRange = new vscode.Range(startPosition, endPosition);
                yield vscode.window.activeTextEditor.edit((editBuilder) => {
                    // re-write the old content in the editor view
                    editBuilder.delete(deleteRange);
                    editBuilder.insert(startPosition, oldDocText);
                });
                yield vscode.window.activeTextEditor.document.save();
            }
            else {
                vscode.window.showErrorMessage(uploadResponse.commandResponse);
            }
        }
        catch (err) {
            vscode.window.showErrorMessage(err.message);
        }
    });
}
exports.saveFile = saveFile;
/**
 * Opens the next tab in editor with given delay
 */
function openNextTab(delay) {
    return new Promise((resolve) => {
        vscode.commands.executeCommand("workbench.action.nextEditor");
        setTimeout(() => resolve(), delay);
    });
}
/**
 * Closes opened file tab using iteration through the tabs
 * This kind of method is caused by incompleteness of VSCode API, which allows to close only currently selected editor
 * For us it means we need to select editor first, which is again not possible via existing VSCode APIs
 */
function closeOpenedTextFile(filePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const tabSwitchDelay = 200;
        const openedWindows = [];
        let selectedEditor = vscode.window.activeTextEditor;
        while (selectedEditor && !openedWindows.some((window) => window.id === selectedEditor.id)) {
            openedWindows.push(selectedEditor);
            yield openNextTab(tabSwitchDelay);
            selectedEditor = vscode.window.activeTextEditor;
            if (selectedEditor && selectedEditor.document.fileName === filePath) {
                vscode.commands.executeCommand("workbench.action.closeActiveEditor");
                return true;
            }
        }
        return false;
    });
}
exports.closeOpenedTextFile = closeOpenedTextFile;

//# sourceMappingURL=actions.js.map
