"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable-next-line: no-duplicate-imports
const zowe = require("@zowe/cli");
const fs = require("fs");
const path = require("path");
const globals = require("./globals");
const vscode = require("vscode");
const ussActions = require("./uss/actions");
const dsActions = require("./dataset/actions");
const jobActions = require("./job/actions");
const sharedActions = require("./shared/actions");
const fs_extra_1 = require("fs-extra");
const imperative_1 = require("@zowe/imperative");
const DatasetTree_1 = require("./dataset/DatasetTree");
const ZosJobsProvider_1 = require("./job/ZosJobsProvider");
const USSTree_1 = require("./uss/USSTree");
const MvsCommandHandler_1 = require("./command/MvsCommandHandler");
const Profiles_1 = require("./Profiles");
const utils_1 = require("./utils");
const SpoolProvider_1 = require("./SpoolProvider");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const KeytarCredentialManager_1 = require("./KeytarCredentialManager");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * The function that runs when the extension is loaded
 *
 * @export
 * @param {vscode.ExtensionContext} context - Context of vscode at the time that the function is called
 * @returns {Promise<ZoweExplorerApiRegister>}
 */
function activate(context) {
    return __awaiter(this, void 0, void 0, function* () {
        // Get temp folder location from settings
        let preferencesTempPath = vscode.workspace.getConfiguration()
            /* tslint:disable:no-string-literal */
            .get("Zowe-Temp-Folder-Location")["folderPath"];
        // Determine the runtime framework to support special behavior for Theia
        globals.defineGlobals(preferencesTempPath);
        // Call cleanTempDir before continuing
        // this is to handle if the application crashed on a previous execution and
        // VSC didn't get a chance to call our deactivate to cleanup.
        yield deactivate();
        try {
            fs.mkdirSync(globals.ZOWETEMPFOLDER);
            fs.mkdirSync(globals.ZOWE_TMP_FOLDER);
            fs.mkdirSync(globals.USS_DIR);
            fs.mkdirSync(globals.DS_DIR);
        }
        catch (err) {
            yield utils_1.errorHandling(err, null, err.message);
        }
        let datasetProvider;
        let ussFileProvider;
        let jobsProvider;
        try {
            globals.initLogger(context);
            globals.LOG.debug(localize(0, null));
            const keytar = getSecurityModules("keytar");
            if (keytar) {
                KeytarCredentialManager_1.KeytarCredentialManager.keytar = keytar;
                const service = vscode.workspace.getConfiguration().get("Zowe Security: Credential Key");
                try {
                    yield imperative_1.CredentialManagerFactory.initialize({
                        service: service || "Zowe-Plugin",
                        Manager: KeytarCredentialManager_1.KeytarCredentialManager,
                        displayName: localize(1, null)
                    });
                }
                catch (err) {
                    throw new imperative_1.ImperativeError({ msg: err.toString() });
                }
            }
            // Ensure that ~/.zowe folder exists
            yield imperative_1.CliProfileManager.initialize({
                configuration: zowe.getImperativeConfig().profiles,
                profileRootDirectory: path.join(utils_1.getZoweDir(), "profiles"),
            });
            // Initialize profile manager
            yield Profiles_1.Profiles.createInstance(globals.LOG);
            // Initialize dataset provider
            datasetProvider = yield DatasetTree_1.createDatasetTree(globals.LOG);
            // Initialize uss provider
            ussFileProvider = yield USSTree_1.createUSSTree(globals.LOG);
            // Initialize Jobs provider with the created session and the selected pattern
            jobsProvider = yield ZosJobsProvider_1.createJobsTree(globals.LOG);
        }
        catch (err) {
            yield utils_1.errorHandling(err, null, (localize(2, null)));
            globals.LOG.error(localize(3, null) + JSON.stringify(err));
        }
        const spoolProvider = new SpoolProvider_1.default();
        const providerRegistration = vscode.Disposable.from(vscode.workspace.registerTextDocumentContentProvider(SpoolProvider_1.default.scheme, spoolProvider));
        context.subscriptions.push(spoolProvider, providerRegistration);
        // Register functions & event listeners
        vscode.workspace.onDidChangeConfiguration((e) => {
            // If the temp folder location has been changed, update current temp folder preference
            if (e.affectsConfiguration("Zowe-Temp-Folder-Location")) {
                const updatedPreferencesTempPath = vscode.workspace.getConfiguration()
                    /* tslint:disable:no-string-literal */
                    .get("Zowe-Temp-Folder-Location")["folderPath"];
                moveTempFolder(preferencesTempPath, updatedPreferencesTempPath);
                preferencesTempPath = updatedPreferencesTempPath;
            }
        });
        if (datasetProvider) {
            initDatasetProvider(context, datasetProvider);
        }
        if (ussFileProvider) {
            initUSSProvider(context, ussFileProvider);
        }
        if (jobsProvider) {
            initJobsProvider(context, jobsProvider);
        }
        if (datasetProvider || ussFileProvider) {
            vscode.commands.registerCommand("zowe.openRecentMember", () => sharedActions.openRecentMemberPrompt(datasetProvider, ussFileProvider));
            vscode.commands.registerCommand("zowe.searchInAllLoadedItems", () => __awaiter(this, void 0, void 0, function* () { return sharedActions.searchInAllLoadedItems(datasetProvider, ussFileProvider); }));
            vscode.workspace.onDidSaveTextDocument((savedFile) => __awaiter(this, void 0, void 0, function* () {
                globals.LOG.debug(localize(4, null) +
                    savedFile.fileName +
                    localize(5, null) +
                    globals.DS_DIR + localize(6, null) + globals.USS_DIR);
                if (savedFile.fileName.toUpperCase().indexOf(globals.DS_DIR.toUpperCase()) >= 0) {
                    globals.LOG.debug(localize(7, null));
                    yield dsActions.saveFile(savedFile, datasetProvider); // TODO MISSED TESTING
                }
                else if (savedFile.fileName.toUpperCase().indexOf(globals.USS_DIR.toUpperCase()) >= 0) {
                    globals.LOG.debug(localize(8, null));
                    yield ussActions.saveUSSFile(savedFile, ussFileProvider); // TODO MISSED TESTING
                }
                else {
                    globals.LOG.debug(localize(9, null) + savedFile.fileName +
                        localize(10, null));
                }
            }));
        }
        // return the Extension's API to other extensions that want to register their APIs.
        return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance();
    });
}
exports.activate = activate;
function initDatasetProvider(context, datasetProvider) {
    vscode.commands.registerCommand("zowe.addSession", () => __awaiter(this, void 0, void 0, function* () { return addZoweSession(datasetProvider); }));
    vscode.commands.registerCommand("zowe.addFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.addFavorite(node); }));
    vscode.commands.registerCommand("zowe.refreshAll", () => dsActions.refreshAll(datasetProvider));
    vscode.commands.registerCommand("zowe.refreshNode", (node) => dsActions.refreshPS(node));
    vscode.commands.registerCommand("zowe.pattern", (node) => datasetProvider.filterPrompt(node));
    vscode.commands.registerCommand("zowe.ZoweNode.openPS", (node) => dsActions.openPS(node, true, datasetProvider));
    vscode.commands.registerCommand("zowe.createDataset", (node) => dsActions.createFile(node, datasetProvider));
    vscode.commands.registerCommand("zowe.createMember", (node) => dsActions.createMember(node, datasetProvider));
    vscode.commands.registerCommand("zowe.deleteDataset", (node) => dsActions.deleteDataset(node, datasetProvider));
    vscode.commands.registerCommand("zowe.deletePDS", (node) => dsActions.deleteDataset(node, datasetProvider));
    vscode.commands.registerCommand("zowe.uploadDialog", (node) => dsActions.uploadDialog(node, datasetProvider));
    vscode.commands.registerCommand("zowe.deleteMember", (node) => dsActions.deleteDataset(node, datasetProvider));
    vscode.commands.registerCommand("zowe.editMember", (node) => dsActions.openPS(node, false, datasetProvider));
    vscode.commands.registerCommand("zowe.removeSession", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.deleteSession(node); }));
    vscode.commands.registerCommand("zowe.removeFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.removeFavorite(node); }));
    vscode.commands.registerCommand("zowe.saveSearch", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.addFavorite(node); }));
    vscode.commands.registerCommand("zowe.removeSavedSearch", (node) => __awaiter(this, void 0, void 0, function* () { return datasetProvider.removeFavorite(node); }));
    vscode.commands.registerCommand("zowe.submitJcl", () => __awaiter(this, void 0, void 0, function* () { return dsActions.submitJcl(datasetProvider); }));
    vscode.commands.registerCommand("zowe.submitMember", (node) => __awaiter(this, void 0, void 0, function* () { return dsActions.submitMember(node); }));
    vscode.commands.registerCommand("zowe.showDSAttributes", (node) => dsActions.showDSAttributes(node, datasetProvider));
    vscode.commands.registerCommand("zowe.renameDataSet", (node) => datasetProvider.rename(node));
    vscode.commands.registerCommand("zowe.copyDataSet", (node) => dsActions.copyDataSet(node));
    vscode.commands.registerCommand("zowe.pasteDataSet", (node) => dsActions.pasteDataSet(node, datasetProvider));
    vscode.commands.registerCommand("zowe.renameDataSetMember", (node) => datasetProvider.rename(node));
    vscode.commands.registerCommand("zowe.hMigrateDataSet", (node) => dsActions.hMigrateDataSet(node));
    vscode.workspace.onDidChangeConfiguration((e) => __awaiter(this, void 0, void 0, function* () { datasetProvider.onDidChangeConfiguration(e); }));
    initSubscribers(context, datasetProvider);
}
function initUSSProvider(context, ussFileProvider) {
    vscode.commands.registerCommand("zowe.uss.addFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.addFavorite(node); }));
    vscode.commands.registerCommand("zowe.uss.removeFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.removeFavorite(node); }));
    vscode.commands.registerCommand("zowe.uss.addSession", () => __awaiter(this, void 0, void 0, function* () { return addZoweSession(ussFileProvider); }));
    vscode.commands.registerCommand("zowe.uss.refreshAll", () => ussActions.refreshAllUSS(ussFileProvider));
    vscode.commands.registerCommand("zowe.uss.refreshUSS", (node) => node.refreshUSS());
    vscode.commands.registerCommand("zowe.uss.refreshUSSInTree", (node) => ussActions.refreshUSSInTree(node, ussFileProvider));
    vscode.commands.registerCommand("zowe.uss.fullPath", (node) => ussFileProvider.filterPrompt(node));
    vscode.commands.registerCommand("zowe.uss.ZoweUSSNode.open", (node) => node.openUSS(false, true, ussFileProvider));
    vscode.commands.registerCommand("zowe.uss.removeSession", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.deleteSession(node); }));
    vscode.commands.registerCommand("zowe.uss.createFile", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.createUSSNode(node, ussFileProvider, "file"); }));
    vscode.commands.registerCommand("zowe.uss.createFolder", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.createUSSNode(node, ussFileProvider, "directory"); }));
    vscode.commands.registerCommand("zowe.uss.deleteNode", (node) => __awaiter(this, void 0, void 0, function* () { return node.deleteUSSNode(ussFileProvider, node.getUSSDocumentFilePath()); }));
    vscode.commands.registerCommand("zowe.uss.binary", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.changeFileType(node, true, ussFileProvider); }));
    vscode.commands.registerCommand("zowe.uss.text", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.changeFileType(node, false, ussFileProvider); }));
    vscode.commands.registerCommand("zowe.uss.renameNode", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.rename(node); }));
    vscode.commands.registerCommand("zowe.uss.uploadDialog", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.uploadDialog(node, ussFileProvider); }));
    vscode.commands.registerCommand("zowe.uss.createNode", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.createUSSNodeDialog(node, ussFileProvider); }));
    vscode.commands.registerCommand("zowe.uss.copyPath", (node) => __awaiter(this, void 0, void 0, function* () { return ussActions.copyPath(node); }));
    vscode.commands.registerCommand("zowe.uss.editFile", (node) => node.openUSS(false, false, ussFileProvider));
    vscode.commands.registerCommand("zowe.uss.saveSearch", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.saveSearch(node); }));
    vscode.commands.registerCommand("zowe.uss.removeSavedSearch", (node) => __awaiter(this, void 0, void 0, function* () { return ussFileProvider.removeFavorite(node); }));
    vscode.workspace.onDidChangeConfiguration((e) => __awaiter(this, void 0, void 0, function* () { ussFileProvider.onDidChangeConfiguration(e); }));
    initSubscribers(context, ussFileProvider);
}
function initJobsProvider(context, jobsProvider) {
    vscode.commands.registerCommand("zowe.zosJobsOpenspool", (session, spool) => jobActions.getSpoolContent(jobsProvider, session, spool));
    vscode.commands.registerCommand("zowe.deleteJob", (job) => jobsProvider.delete(job));
    vscode.commands.registerCommand("zowe.runModifyCommand", (job) => jobActions.modifyCommand(job));
    vscode.commands.registerCommand("zowe.runStopCommand", (job) => jobActions.stopCommand(job));
    vscode.commands.registerCommand("zowe.refreshJobsServer", (job) => __awaiter(this, void 0, void 0, function* () { return jobActions.refreshJobsServer(job, jobsProvider); }));
    vscode.commands.registerCommand("zowe.refreshAllJobs", () => __awaiter(this, void 0, void 0, function* () { return jobActions.refreshAllJobs(jobsProvider); }));
    vscode.commands.registerCommand("zowe.addJobsSession", () => addZoweSession(jobsProvider));
    vscode.commands.registerCommand("zowe.setOwner", (job) => jobActions.setOwner(job, jobsProvider));
    vscode.commands.registerCommand("zowe.setPrefix", (job) => jobActions.setPrefix(job, jobsProvider));
    vscode.commands.registerCommand("zowe.removeJobsSession", (job) => jobsProvider.deleteSession(job));
    vscode.commands.registerCommand("zowe.downloadSpool", (job) => jobActions.downloadSpool(job));
    vscode.commands.registerCommand("zowe.getJobJcl", (job) => jobActions.downloadJcl(job));
    vscode.commands.registerCommand("zowe.setJobSpool", (session, jobid) => __awaiter(this, void 0, void 0, function* () {
        const sessionNode = jobsProvider.mSessionNodes.find((jobNode) => jobNode.label.trim() === session.trim());
        sessionNode.dirty = true;
        jobsProvider.refresh();
        const jobs = yield sessionNode.getChildren();
        const job = jobs.find((jobNode) => jobNode.job.jobid === jobid);
        jobsProvider.setItem(jobsProvider.getTreeView(), job);
    }));
    vscode.commands.registerCommand("zowe.jobs.search", (node) => jobsProvider.filterPrompt(node));
    vscode.commands.registerCommand("zowe.issueTsoCmd", () => __awaiter(this, void 0, void 0, function* () { return MvsCommandHandler_1.MvsCommandHandler.getInstance().issueMvsCommand(); }));
    vscode.commands.registerCommand("zowe.issueMvsCmd", (node, command) => __awaiter(this, void 0, void 0, function* () { return MvsCommandHandler_1.MvsCommandHandler.getInstance().issueMvsCommand(node.session, command); }));
    vscode.commands.registerCommand("zowe.jobs.addFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.addFavorite(node); }));
    vscode.commands.registerCommand("zowe.jobs.removeFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.removeFavorite(node); }));
    vscode.commands.registerCommand("zowe.jobs.saveSearch", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.saveSearch(node); }));
    vscode.commands.registerCommand("zowe.jobs.removeSearchFavorite", (node) => __awaiter(this, void 0, void 0, function* () { return jobsProvider.removeFavorite(node); }));
    vscode.workspace.onDidChangeConfiguration((e) => __awaiter(this, void 0, void 0, function* () { jobsProvider.onDidChangeConfiguration(e); }));
    initSubscribers(context, jobsProvider);
}
function initSubscribers(context, theProvider) {
    const theTreeView = theProvider.getTreeView();
    context.subscriptions.push(theTreeView);
    if (!globals.ISTHEIA) {
        theTreeView.onDidCollapseElement((e) => __awaiter(this, void 0, void 0, function* () { yield theProvider.flipState(e.element, false); }));
        theTreeView.onDidExpandElement((e) => __awaiter(this, void 0, void 0, function* () { yield theProvider.flipState(e.element, true); }));
    }
}
/**
 * function to check if imperative.json contains
 * information about security or not and then
 * Imports the neccesary security modules
 */
function getSecurityModules(moduleName) {
    let imperativeIsSecure = false;
    try {
        const fileName = path.join(utils_1.getZoweDir(), "settings", "imperative.json");
        let settings;
        if (fs.existsSync(fileName)) {
            settings = JSON.parse(fs.readFileSync(fileName).toString());
        }
        const value1 = settings === null || settings === void 0 ? void 0 : settings.overrides.CredentialManager;
        const value2 = settings === null || settings === void 0 ? void 0 : settings.overrides["credential-manager"];
        imperativeIsSecure = ((typeof value1 === "string") && (value1.length > 0)) ||
            ((typeof value2 === "string") && (value2.length > 0));
    }
    catch (error) {
        globals.LOG.warn(localize(11, null) + error.message);
        vscode.window.showWarningMessage(error.message);
        return undefined;
    }
    if (imperativeIsSecure) {
        // Workaround for Theia issue (https://github.com/eclipse-theia/theia/issues/4935)
        const appRoot = globals.ISTHEIA ? process.cwd() : vscode.env.appRoot;
        try {
            return require(`${appRoot}/node_modules/${moduleName}`);
        }
        catch (err) { /* Do nothing */ }
        try {
            return require(`${appRoot}/node_modules.asar/${moduleName}`);
        }
        catch (err) { /* Do nothing */ }
        vscode.window.showWarningMessage(localize(12, null) + moduleName);
    }
    return undefined;
}
exports.getSecurityModules = getSecurityModules;
/**
 * Moves temp folder to user defined location in preferences
 * @param previousTempPath temp path settings value before updated by user
 * @param currentTempPath temp path settings value after updated by user
 */
function moveTempFolder(previousTempPath, currentTempPath) {
    // Re-define globals with updated path
    globals.defineGlobals(currentTempPath);
    if (previousTempPath === "") {
        previousTempPath = path.join(__dirname, "..", "..", "resources");
    }
    // Make certain that "temp" folder is cleared
    cleanTempDir();
    try {
        fs.mkdirSync(globals.ZOWETEMPFOLDER);
        fs.mkdirSync(globals.ZOWE_TMP_FOLDER);
        fs.mkdirSync(globals.USS_DIR);
        fs.mkdirSync(globals.DS_DIR);
    }
    catch (err) {
        globals.LOG.error(localize(13, null) + JSON.stringify(err));
        utils_1.errorHandling(err, null, localize(14, null) + err.message);
    }
    const previousTemp = path.join(previousTempPath, "temp");
    try {
        // If source and destination path are same, exit
        if (previousTemp === globals.ZOWETEMPFOLDER) {
            return;
        }
        // TODO: Possibly remove when supporting "Multiple Instances"
        // If a second instance has already moved the temp folder, exit
        // Ideally, `moveSync()` would alert user if path doesn't exist.
        // However when supporting "Multiple Instances", might not be possible.
        if (!fs.existsSync(previousTemp)) {
            return;
        }
        fs_extra_1.moveSync(previousTemp, globals.ZOWETEMPFOLDER, { overwrite: true });
    }
    catch (err) {
        globals.LOG.error("Error moving temporary folder! " + JSON.stringify(err));
        vscode.window.showErrorMessage(err.message);
    }
}
exports.moveTempFolder = moveTempFolder;
/**
 * Adds a new Profile to the provided treeview by clicking the 'Plus' button and
 * selecting which profile you would like to add from the drop-down that appears.
 * The profiles that are in the tree view already will not appear in the
 * drop-down.
 *
 * @export
 * @param {USSTree} zoweFileProvider - either the USS, MVS, JES tree
 */
function addZoweSession(zoweFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        const allProfiles = (yield Profiles_1.Profiles.getInstance()).allProfiles;
        const createNewProfile = "Create a New Connection to z/OS";
        let chosenProfile = "";
        // Get all profiles
        let profileNamesList = allProfiles.map((profile) => {
            return profile.name;
        });
        // Filter to list of the APIs available for current tree explorer
        profileNamesList = profileNamesList.filter((profileName) => {
            const profile = Profiles_1.Profiles.getInstance().loadNamedProfile(profileName);
            if (zoweFileProvider instanceof USSTree_1.USSTree) {
                const ussProfileTypes = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredUssApiTypes();
                return ussProfileTypes.includes(profile.type);
            }
            if (zoweFileProvider instanceof DatasetTree_1.DatasetTree) {
                const mvsProfileTypes = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredMvsApiTypes();
                return mvsProfileTypes.includes(profile.type);
            }
            if (zoweFileProvider instanceof ZosJobsProvider_1.ZosJobsProvider) {
                const jesProfileTypes = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredJesApiTypes();
                return jesProfileTypes.includes(profile.type);
            }
        });
        if (profileNamesList) {
            profileNamesList = profileNamesList.filter((profileName) => 
            // Find all cases where a profile is not already displayed
            !zoweFileProvider.mSessionNodes.find((sessionNode) => sessionNode.getProfileName() === profileName));
        }
        const createPick = new utils_1.FilterDescriptor("\uFF0B " + createNewProfile);
        const items = profileNamesList.map((element) => new utils_1.FilterItem(element));
        const quickpick = vscode.window.createQuickPick();
        const placeholder = localize(15, null);
        if (globals.ISTHEIA) {
            const options = {
                placeHolder: placeholder
            };
            // get user selection
            const choice = (yield vscode.window.showQuickPick([createPick, ...items], options));
            if (!choice) {
                vscode.window.showInformationMessage(localize(16, null));
                return;
            }
            chosenProfile = choice === createPick ? "" : choice.label;
        }
        else {
            quickpick.items = [createPick, ...items];
            quickpick.placeholder = placeholder;
            quickpick.ignoreFocusOut = true;
            quickpick.show();
            const choice = yield utils_1.resolveQuickPickHelper(quickpick);
            quickpick.hide();
            if (!choice) {
                vscode.window.showInformationMessage(localize(17, null));
                return;
            }
            if (choice instanceof utils_1.FilterDescriptor) {
                chosenProfile = "";
            }
            else {
                chosenProfile = choice.label;
            }
        }
        if (chosenProfile === "") {
            let newprofile;
            let profileName;
            if (quickpick.value) {
                profileName = quickpick.value;
            }
            const options = {
                placeHolder: localize(18, null),
                prompt: localize(19, null),
                value: profileName
            };
            profileName = yield vscode.window.showInputBox(options);
            if (!profileName) {
                vscode.window.showInformationMessage(localize(20, null));
                return;
            }
            chosenProfile = profileName.trim();
            globals.LOG.debug(localize(21, null));
            try {
                newprofile = yield Profiles_1.Profiles.getInstance().createNewConnection(chosenProfile);
            }
            catch (error) {
                yield utils_1.errorHandling(error, chosenProfile, error.message);
            }
            if (newprofile) {
                try {
                    yield Profiles_1.Profiles.getInstance().refresh();
                }
                catch (error) {
                    yield utils_1.errorHandling(error, newprofile, error.message);
                }
                yield zoweFileProvider.addSession(newprofile);
                yield zoweFileProvider.refresh();
            }
        }
        else if (chosenProfile) {
            globals.LOG.debug(localize(22, null) + chosenProfile);
            yield zoweFileProvider.addSession(chosenProfile);
        }
        else {
            globals.LOG.debug(localize(23, null));
        }
    });
}
exports.addZoweSession = addZoweSession;
/**
 * Recursively deletes directory
 *
 * @param directory path to directory to be deleted
 */
function cleanDir(directory) {
    if (!fs.existsSync(directory)) {
        return;
    }
    fs.readdirSync(directory).forEach((file) => {
        const fullpath = path.join(directory, file);
        const lstat = fs.lstatSync(fullpath);
        if (lstat.isFile()) {
            fs.unlinkSync(fullpath);
        }
        else {
            cleanDir(fullpath);
        }
    });
    fs.rmdirSync(directory);
}
exports.cleanDir = cleanDir;
/**
 * Cleans up local temp directory
 *
 * @export
 */
function cleanTempDir() {
    return __awaiter(this, void 0, void 0, function* () {
        // logger hasn't necessarily been initialized yet, don't use the `log` in this function
        if (!fs.existsSync(globals.ZOWETEMPFOLDER)) {
            return;
        }
        try {
            cleanDir(globals.ZOWETEMPFOLDER);
        }
        catch (err) {
            vscode.window.showErrorMessage(localize(24, null) + err);
        }
    });
}
exports.cleanTempDir = cleanTempDir;
/**
 * Called by VSCode on shutdown
 *
 * @export
 */
function deactivate() {
    return __awaiter(this, void 0, void 0, function* () {
        yield cleanTempDir();
    });
}
exports.deactivate = deactivate;

//# sourceMappingURL=extension.js.map
