"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const globals = require("../globals");
const actions_1 = require("../dataset/actions");
const utils_1 = require("../shared/utils");
const utils_2 = require("../utils");
const contextually = require("../shared/context");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Search for matching items loaded in data set or USS tree
 *
 */
function searchInAllLoadedItems(datasetProvider, ussFileProvider) {
    return __awaiter(this, void 0, void 0, function* () {
        let pattern;
        const items = [];
        const qpItems = [];
        const quickpick = vscode.window.createQuickPick();
        quickpick.placeholder = localize(0, null);
        quickpick.ignoreFocusOut = true;
        quickpick.onDidChangeValue((value) => __awaiter(this, void 0, void 0, function* () {
            if (value) {
                quickpick.items = utils_1.filterTreeByString(value, qpItems);
            }
            else {
                quickpick.items = [...qpItems];
            }
        }));
        // Get loaded items from Tree Providers
        if (datasetProvider) {
            const newItems = yield datasetProvider.searchInLoadedItems();
            items.push(...newItems);
        }
        if (ussFileProvider) {
            const newItems = yield ussFileProvider.searchInLoadedItems();
            items.push(...newItems);
        }
        if (items.length === 0) {
            vscode.window.showInformationMessage(localize(1, null));
            return;
        }
        let qpItem;
        for (const item of items) {
            if (item.constructor.name === "ZoweDatasetNode") {
                if (contextually.isDsMember(item)) {
                    qpItem = new utils_2.FilterItem(`[${item.getSessionNode().label.trim()}]: ${item.getParent().label.trim()}(${item.label.trim()})`, "Data Set Member");
                }
                else {
                    qpItem = new utils_2.FilterItem(`[${item.getSessionNode().label.trim()}]: ${item.label.trim()}`, "Data Set");
                }
                qpItems.push(qpItem);
            }
            else if (item.constructor.name === "ZoweUSSNode") {
                const filterItem = `[${item.getProfileName().trim()}]: ${item.getParent().fullPath}/${item.label.trim()}`;
                qpItem = new utils_2.FilterItem(filterItem, "USS");
                qpItems.push(qpItem);
            }
        }
        quickpick.items = [...qpItems];
        quickpick.show();
        const choice = yield utils_2.resolveQuickPickHelper(quickpick);
        if (!choice) {
            vscode.window.showInformationMessage(localize(2, null));
            return;
        }
        else {
            pattern = choice.label;
        }
        quickpick.dispose();
        if (pattern) {
            // Parse pattern for item name
            let filePath;
            let nodeName;
            let memberName;
            const sessionName = pattern.substring(1, pattern.indexOf("]"));
            if (pattern.indexOf("(") !== -1) {
                nodeName = pattern.substring(pattern.indexOf(" ") + 1, pattern.indexOf("("));
                memberName = pattern.substring(pattern.indexOf("(") + 1, pattern.indexOf(")"));
            }
            else if (pattern.indexOf("/") !== -1) {
                filePath = pattern.substring(pattern.indexOf(" ") + 1);
            }
            else {
                nodeName = pattern.substring(pattern.indexOf(" ") + 1);
            }
            // Find & reveal nodes in tree
            if (pattern.indexOf("/") !== -1) {
                // USS nodes
                const node = items.filter((item) => item.fullPath.trim() === filePath)[0];
                ussFileProvider.setItem(ussFileProvider.getTreeView(), node);
                if (node.contextValue !== globals.USS_DIR_CONTEXT) {
                    // If selected item is file, open it in workspace
                    ussFileProvider.addHistory(node.fullPath);
                    const ussNode = node;
                    ussNode.openUSS(false, true, ussFileProvider);
                }
            }
            else {
                // Data set nodes
                const sessions = yield datasetProvider.getChildren();
                const sessionNode = sessions.filter((session) => session.label.trim() === sessionName)[0];
                let children = yield datasetProvider.getChildren(sessionNode);
                const node = children.filter((child) => child.label.trim() === nodeName)[0];
                if (memberName) {
                    // Members
                    children = yield datasetProvider.getChildren(node);
                    const member = children.filter((child) => child.label.trim() === memberName)[0];
                    node.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                    yield datasetProvider.getTreeView().reveal(member, { select: true, focus: true, expand: false });
                    // Open in workspace
                    datasetProvider.addHistory(`${nodeName}(${memberName})`);
                    actions_1.openPS(member, true, datasetProvider);
                }
                else {
                    // PDS & SDS
                    yield datasetProvider.getTreeView().reveal(node, { select: true, focus: true, expand: false });
                    // If selected node was SDS, open it in workspace
                    if (contextually.isDs(node)) {
                        datasetProvider.addHistory(nodeName);
                        actions_1.openPS(node, true, datasetProvider);
                    }
                }
            }
        }
    });
}
exports.searchInAllLoadedItems = searchInAllLoadedItems;
function openRecentMemberPrompt(datasetTree, ussTree) {
    return __awaiter(this, void 0, void 0, function* () {
        if (globals.LOG) {
            globals.LOG.debug(localize(3, null));
        }
        let pattern;
        const allRecall = [...datasetTree.getRecall(), ...ussTree.getRecall()];
        // Get user selection
        if (allRecall.length > 0) {
            const createPick = new utils_2.FilterDescriptor(localize(4, null));
            const items = allRecall.map((element) => new utils_2.FilterItem(element));
            if (globals.ISTHEIA) {
                const options1 = {
                    placeHolder: localize(5, null)
                };
                const choice = (yield vscode.window.showQuickPick([createPick, ...items], options1));
                if (!choice) {
                    vscode.window.showInformationMessage(localize(6, null));
                    return;
                }
                pattern = choice === createPick ? "" : choice.label;
            }
            else {
                const quickpick = vscode.window.createQuickPick();
                quickpick.items = [createPick, ...items];
                quickpick.placeholder = localize(7, null);
                quickpick.ignoreFocusOut = true;
                quickpick.show();
                const choice = yield utils_2.resolveQuickPickHelper(quickpick);
                quickpick.hide();
                if (!choice || choice === createPick) {
                    vscode.window.showInformationMessage(localize(8, null));
                    return;
                }
                else if (choice instanceof utils_2.FilterDescriptor) {
                    if (quickpick.value) {
                        pattern = quickpick.value;
                    }
                }
                else {
                    pattern = choice.label;
                }
            }
            const sessionName = pattern.substring(1, pattern.indexOf("]")).trim();
            if (pattern.indexOf("/") > -1) {
                // USS file was selected
                const filePath = pattern.substring(pattern.indexOf("/"));
                const sessionNode = ussTree.mSessionNodes.find((sessNode) => sessNode.getProfileName() === sessionName);
                yield ussTree.openItemFromPath(filePath, sessionNode);
            }
            else {
                // Data set was selected
                const sessionNode = datasetTree.mSessionNodes.find((sessNode) => sessNode.label.trim() === sessionName);
                yield datasetTree.openItemFromPath(pattern, sessionNode);
            }
        }
        else {
            vscode.window.showInformationMessage(localize(9, null));
            return;
        }
    });
}
exports.openRecentMemberPrompt = openRecentMemberPrompt;

//# sourceMappingURL=actions.js.map
