"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
Object.defineProperty(exports, "__esModule", { value: true });
const globals = require("../globals");
/**
 *
 * The contextValue is made up with the name of the node type as the first value in the sequence.
 * All subsequent attributes of the contextValue are preceded by an underscore character.
 * example:
 * pds_fav represents a pds file indicated as a favorite.
 *
 * Additional fields can be added as underscore values
 * example:
 * job_fav_rc=CC 0000
 *
 */
/**
 * Helper function which identifies if the node is a ds
 * @param node
 * @return true if a ds, false otherwise
 */
function isDs(node) {
    return new RegExp("^" + globals.DS_DS_CONTEXT).test(node.contextValue);
}
exports.isDs = isDs;
/**
 * Helper function which identifies if the node is a job
 * @param node
 * @return true if a job, false otherwise
 */
function isJob(node) {
    return new RegExp("^" + globals.JOBS_JOB_CONTEXT).test(node.contextValue);
}
exports.isJob = isJob;
/**
 * Helper function which identifies if the node is a pds or ds and a favorite
 * @param node
 * @return true if a favorite pds, ds, false otherwise
 */
function isFavoritePsDs(node) {
    return new RegExp("^(" + globals.DS_PDS_CONTEXT + "|" +
        globals.DS_DS_CONTEXT + ")(.*" + globals.FAV_SUFFIX + ")").test(node.contextValue);
}
exports.isFavoritePsDs = isFavoritePsDs;
/**
 * Helper function which identifies if the node is a ds and a favorite
 * @param node
 * @return true if a favorite ds, false otherwise
 */
function isFavoriteDs(node) {
    return new RegExp("^(" + globals.DS_DS_CONTEXT + ")(.*" + globals.FAV_SUFFIX + ")").test(node.contextValue);
}
exports.isFavoriteDs = isFavoriteDs;
/**
 * Helper function which identifies if the node is a job and it's a favorite
 * @param node
 * @return true if a favorite job, false otherwise
 */
function isFavoriteJob(node) {
    return new RegExp("^(" + globals.JOBS_JOB_CONTEXT + ")(.*" + globals.FAV_SUFFIX + ")").test(node.contextValue);
}
exports.isFavoriteJob = isFavoriteJob;
/**
 * Helper function which identifies if the node is a pds and a favorite
 * @param node
 * @return true if a favorite pds, false otherwise
 */
function isFavoritePds(node) {
    return new RegExp("^(" + globals.DS_PDS_CONTEXT + ")(.*" + globals.FAV_SUFFIX + ")").test(node.contextValue);
}
exports.isFavoritePds = isFavoritePds;
/**
 * Helper function which identifies if the node is a Favorite binary or text file
 * @param node
 * @return true if a Favorite binary or text file, false otherwise
 */
function isFavoriteTextOrBinary(node) {
    return new RegExp("^(" + globals.DS_BINARY_FILE_CONTEXT + "|" +
        globals.DS_TEXT_FILE_CONTEXT + ")(.*" + globals.FAV_SUFFIX + ")").test(node.contextValue);
}
exports.isFavoriteTextOrBinary = isFavoriteTextOrBinary;
/**
 * Helper function which identifies if the node is a binary file
 * @param node
 * @return true if a binary file, false otherwise
 */
function isBinary(node) {
    return new RegExp("^" + globals.DS_BINARY_FILE_CONTEXT).test(node.contextValue);
}
exports.isBinary = isBinary;
/**
 * Helper function which identifies if the node is a text file
 * @param node
 * @return true if a text file, false otherwise
 */
function isText(node) {
    return new RegExp("^" + globals.DS_TEXT_FILE_CONTEXT).test(node.contextValue);
}
exports.isText = isText;
/**
 * Helper function which identifies if the node is a document
 * @param node
 * @return true if a document, false otherwise
 */
function isDocument(node) {
    return new RegExp("^(" + globals.DS_DS_CONTEXT + "|" + globals.DS_MEMBER_CONTEXT + "|"
        + globals.DS_TEXT_FILE_CONTEXT + "|" + globals.JOBS_SPOOL_CONTEXT + "|"
        + globals.DS_BINARY_FILE_CONTEXT + ")").test(node.contextValue);
}
exports.isDocument = isDocument;
/**
 * Helper function which identifies if the node is a informational only
 * @param node
 * @return true if a informational, false otherwise
 */
function isInformation(node) {
    return new RegExp("^(" + globals.INFORMATION_CONTEXT + ")").test(node.contextValue);
}
exports.isInformation = isInformation;
/**
 * Helper function which identifies if the node is migrated
 * @param node
 * @return true if a migrated dataset, false otherwise
 */
function isMigrated(node) {
    return new RegExp("^(" + globals.DS_MIGRATED_FILE_CONTEXT + ")").test(node.contextValue);
}
exports.isMigrated = isMigrated;
/**
 * Helper function which identifies if the node is a favorite
 * @param node
 * @return true if a favorite, false otherwise
 */
function isFavorite(node) {
    return new RegExp(globals.FAV_SUFFIX).test(node.contextValue);
}
exports.isFavorite = isFavorite;
/**
 * Helper function which identifies if the node is a favorite root
 * or search
 * @param node
 * @return true if a favorite root, false otherwise
 */
function isFavoriteSearch(node) {
    return new RegExp("^(" + globals.JOBS_SESSION_CONTEXT + "|" + globals.USS_SESSION_CONTEXT + "|"
        + globals.DS_SESSION_CONTEXT + ")(.*" + globals.FAV_SUFFIX + ")").test(node.contextValue);
}
exports.isFavoriteSearch = isFavoriteSearch;
/**
 * Helper function which identifies if the node is a favorite context
 * @param node
 * @return true if a favorite context root, false otherwise
 */
function isFavoriteContext(node) {
    return new RegExp(globals.FAVORITE_CONTEXT).test(node.contextValue);
}
exports.isFavoriteContext = isFavoriteContext;
/**
 * Helper function which identifies if the node is a dataset member
 * @param node
 * @return true if a dataset member, false otherwise
 */
function isDsMember(node) {
    return new RegExp("^(" + globals.DS_MEMBER_CONTEXT + ")").test(node.contextValue);
}
exports.isDsMember = isDsMember;
/**
 * Helper function which identifies if the node is a dataset session
 * @param node
 * @return true if a dataset session, false otherwise
 */
function isDsSession(node) {
    return new RegExp("^(" + globals.DS_SESSION_CONTEXT + ")").test(node.contextValue);
}
exports.isDsSession = isDsSession;
/**
 * Helper function which identifies if the node is a partitioned dataset
 * @param node
 * @return true if a partitioned dataset, false otherwise
 */
function isPdsNotFav(node) {
    return new RegExp("^(?!.*" + globals.FAV_SUFFIX + ")" + globals.DS_PDS_CONTEXT).test(node.contextValue);
}
exports.isPdsNotFav = isPdsNotFav;
/**
 * Helper function which identifies if the node is a USS Directory
 * @param node
 * @return true if a USS Directory, false otherwise
 */
function isUssDirectory(node) {
    return new RegExp("^" + globals.USS_DIR_CONTEXT).test(node.contextValue);
}
exports.isUssDirectory = isUssDirectory;
/**
 * Helper function which identifies if the node is a USS session
 * @param node
 * @return true if a USS session, false otherwise
 */
function isUssSession(node) {
    return new RegExp("^(" + globals.USS_SESSION_CONTEXT + ")").test(node.contextValue);
}
exports.isUssSession = isUssSession;
/**
 * Helper function which identifies if the node is a grouping or folder
 * @param node
 * @return true if a folder, false otherwise
 */
function isFolder(node) {
    return new RegExp("^(" + globals.JOBS_JOB_CONTEXT + "|" + globals.USS_DIR_CONTEXT + "|"
        + globals.DS_PDS_CONTEXT + ")").test(node.contextValue);
}
exports.isFolder = isFolder;
/**
 * Helper function which identifies if the node is a session
 * @param node
 * @return true if a session, false otherwise
 */
function isSession(node) {
    return new RegExp("^(" + globals.JOBS_SESSION_CONTEXT + "|" + globals.USS_SESSION_CONTEXT + "|"
        + globals.DS_SESSION_CONTEXT + ")").test(node.contextValue);
}
exports.isSession = isSession;
/**
 * Helper function which identifies if the node is a session but not a favorite
 * @param node
 * @return true if a session, false otherwise
 */
function isSessionNotFav(node) {
    return new RegExp("^((?!.*" + globals.FAV_SUFFIX + ")(" + globals.JOBS_SESSION_CONTEXT + "|" + globals.USS_SESSION_CONTEXT + "|"
        + globals.DS_SESSION_CONTEXT + "))").test(node.contextValue);
}
exports.isSessionNotFav = isSessionNotFav;
/**
 * Helper function which identifies if the node is a session favorite
 * @param node
 * @return true if a session favorite, false otherwise
 */
function isSessionFavorite(node) {
    return new RegExp("^(" + globals.FAVORITE_CONTEXT + ")").test(node.contextValue);
}
exports.isSessionFavorite = isSessionFavorite;
/**
 * Helper function create the favorite version of a node
 * @param node
 * @return If not a favorite an extended contextValue with _fav.
 * If the value is a favorite already that contextValue is returned.
 */
function asFavorite(node) {
    return isFavorite(node) ? node.contextValue : node.contextValue + globals.FAV_SUFFIX;
}
exports.asFavorite = asFavorite;
/**
 * Helper function to retrieve the base context of a node
 * @param node
 * @return The inital element of the context.
 */
function getBaseContext(node) {
    return node.contextValue.indexOf(globals.CONTEXT_PREFIX) > -1 ?
        node.contextValue.substring(0, node.contextValue.indexOf(globals.CONTEXT_PREFIX)) :
        node.contextValue;
}
exports.getBaseContext = getBaseContext;

//# sourceMappingURL=context.js.map
