"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const globals = require("../globals");
const path = require("path");
const utils_1 = require("../utils");
const utils_2 = require("../shared/utils");
const ZoweUSSNode_1 = require("./ZoweUSSNode");
const Profiles_1 = require("../Profiles");
const ZoweTreeProvider_1 = require("../abstract/ZoweTreeProvider");
const ZoweExplorerApiRegister_1 = require("../api/ZoweExplorerApiRegister");
const icons_1 = require("../generators/icons");
const contextually = require("../shared/context");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * Creates the USS tree that contains nodes of sessions and data sets
 *
 * @export
 */
function createUSSTree(log) {
    return __awaiter(this, void 0, void 0, function* () {
        const tree = new USSTree();
        yield tree.initialize(log);
        yield tree.addSession();
        return tree;
    });
}
exports.createUSSTree = createUSSTree;
/**
 * A tree that contains nodes of sessions and USS Files
 *
 * @export
 * @class USSTree
 * @implements {vscode.TreeDataProvider}
 */
class USSTree extends ZoweTreeProvider_1.ZoweTreeProvider {
    constructor() {
        super(USSTree.persistenceSchema, new ZoweUSSNode_1.ZoweUSSNode(localize(0, null), vscode.TreeItemCollapsibleState.Collapsed, null, null, null));
        this.mSessionNodes = [];
        this.mFavorites = [];
        this.mFavoriteSession.contextValue = globals.FAVORITE_CONTEXT;
        const icon = icons_1.getIconByNode(this.mFavoriteSession);
        if (icon) {
            this.mFavoriteSession.iconPath = icon.path;
        }
        this.mSessionNodes = [this.mFavoriteSession];
        this.treeView = vscode.window.createTreeView("zowe.uss.explorer", { treeDataProvider: this });
    }
    /**
     * Method for renaming a USS Node. This could be a Favorite Node
     *
     * @param originalNode
     * @param {string} filePath
     */
    rename(originalNode) {
        return __awaiter(this, void 0, void 0, function* () {
            // Could be a favorite or regular entry always deal with the regular entry
            const oldLabel = originalNode.label;
            const parentPath = originalNode.fullPath.substr(0, originalNode.fullPath.indexOf(oldLabel));
            // Check if an old favorite exists for this node
            const oldFavorite = contextually.isFavorite(originalNode) ? originalNode : this.mFavorites.find((temp) => (temp.shortLabel === oldLabel) && (temp.fullPath.substr(0, temp.fullPath.indexOf(oldLabel)) === parentPath));
            const newName = yield vscode.window.showInputBox({ value: oldLabel });
            if (newName && newName !== oldLabel) {
                try {
                    let newNamePath = path.join(parentPath + newName);
                    newNamePath = newNamePath.replace(/\\/g, "/"); // Added to cover Windows backslash issue
                    yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(originalNode.getProfile()).rename(originalNode.fullPath, newNamePath);
                    originalNode.rename(newNamePath);
                    if (oldFavorite) {
                        this.removeFavorite(oldFavorite);
                        oldFavorite.rename(newNamePath);
                        this.addFavorite(oldFavorite);
                    }
                }
                catch (err) {
                    utils_1.errorHandling(err, originalNode.mProfileName, localize(1, null) + err.message);
                    throw (err);
                }
            }
        });
    }
    open(node, preview) {
        throw new Error("Method not implemented.");
    }
    copy(node) {
        throw new Error("Method not implemented.");
    }
    paste(node) {
        throw new Error("Method not implemented.");
    }
    delete(node) {
        throw new Error("Method not implemented.");
    }
    saveFile(document) {
        throw new Error("Method not implemented.");
    }
    refreshPS(node) {
        throw new Error("Method not implemented.");
    }
    uploadDialog(node) {
        throw new Error("Method not implemented.");
    }
    /**
     * Returns the tree view for the current USSTree
     *
     * @returns {vscode.TreeView<ZoweUSSNode>}
     */
    getTreeView() {
        return this.treeView;
    }
    /**
     * Takes argument of type IZoweUSSTreeNode and retrieves all of the first level children
     *
     * @param {IZoweUSSTreeNode} [element] - Optional parameter; if not passed, returns root session nodes
     * @returns {IZoweUSSTreeNode[] | Promise<IZoweUSSTreeNode[]>}
     */
    getChildren(element) {
        return __awaiter(this, void 0, void 0, function* () {
            if (element) {
                if (contextually.isFavoriteContext(element)) {
                    return this.mFavorites;
                }
                yield Profiles_1.Profiles.getInstance().checkCurrentProfile(element.getProfile());
                return element.getChildren();
            }
            return this.mSessionNodes;
        });
    }
    /**
     * Adds a new session to the uss files tree
     *
     * @param {string} [sessionName] - optional; loads persisted profiles or default if not passed
     */
    addSession(sessionName) {
        return __awaiter(this, void 0, void 0, function* () {
            // Loads profile associated with passed sessionName, persisted profiles or default if none passed
            if (sessionName) {
                const profile = Profiles_1.Profiles.getInstance().loadNamedProfile(sessionName);
                if (profile) {
                    this.addSingleSession(profile);
                }
            }
            else {
                const allProfiles = Profiles_1.Profiles.getInstance().allProfiles;
                for (const profile of allProfiles) {
                    // If session is already added, do nothing
                    if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === profile.name)) {
                        continue;
                    }
                    for (const session of this.mHistory.getSessions()) {
                        if (session === profile.name) {
                            this.addSingleSession(profile);
                        }
                    }
                }
                if (this.mSessionNodes.length === 1) {
                    this.addSingleSession(Profiles_1.Profiles.getInstance().getDefaultProfile());
                }
            }
            this.refresh();
        });
    }
    /**
     * Removes a session from the list in the uss files tree
     *
     * @param {IZoweUSSTreeNode} [node]
     */
    deleteSession(node) {
        this.mSessionNodes = this.mSessionNodes.filter((tempNode) => tempNode.label.trim() !== node.label.trim());
        let revisedLabel = node.label;
        if (revisedLabel.includes("[")) {
            revisedLabel = revisedLabel.substring(0, revisedLabel.indexOf(" ["));
        }
        this.mHistory.removeSession(revisedLabel);
        this.refresh();
    }
    /**
     * Adds a node to the USS favorites list
     *
     * @param {IZoweUSSTreeNode} node
     */
    addFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            let temp;
            temp = new ZoweUSSNode_1.ZoweUSSNode(node.fullPath, node.collapsibleState, this.mFavoriteSession, node.getSession(), node.getParent().fullPath, false, node.getSessionNode().getProfileName());
            temp.contextValue = contextually.asFavorite(temp);
            if (contextually.isFavoriteTextOrBinary(temp)) {
                temp.command = { command: "zowe.uss.ZoweUSSNode.open", title: "Open", arguments: [temp] };
            }
            const icon = icons_1.getIconByNode(temp);
            if (icon) {
                temp.iconPath = icon.path;
            }
            if (!this.mFavorites.find((tempNode) => tempNode.label === temp.label)) {
                this.mFavorites.push(temp);
                utils_2.sortTreeItems(this.mFavorites, globals.USS_SESSION_CONTEXT + globals.FAV_SUFFIX);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Adds a search node to the USS favorites list
     *
     * @param {IZoweUSSTreeNode} node
     */
    saveSearch(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const label = "[" + node.getSessionNode().getProfileName() + "]: " + node.fullPath;
            const temp = new ZoweUSSNode_1.ZoweUSSNode(label, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, node.getSession(), null, false, node.getSessionNode().getProfileName());
            temp.fullPath = node.fullPath;
            temp.label = temp.tooltip = label;
            temp.contextValue = globals.USS_SESSION_CONTEXT + globals.FAV_SUFFIX;
            const icon = icons_1.getIconByNode(temp);
            if (icon) {
                temp.iconPath = icon.path;
            }
            temp.command = { command: "zowe.uss.fullPath", title: "", arguments: [temp] };
            if (!this.mFavorites.find((tempNode) => tempNode.label === temp.label)) {
                this.mFavorites.push(temp);
                utils_2.sortTreeItems(this.mFavorites, globals.USS_SESSION_CONTEXT + globals.FAV_SUFFIX);
                yield this.updateFavorites();
                this.refreshElement(this.mFavoriteSession);
            }
        });
    }
    /**
     * Removes a node from the favorites list
     *
     * @param {IZoweUSSTreeNode} node
     */
    removeFavorite(node) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mFavorites = this.mFavorites.filter((temp) => !((temp.fullPath === node.fullPath) && (temp.contextValue.startsWith(node.contextValue))));
            yield this.updateFavorites();
            this.refreshElement(this.mFavoriteSession);
        });
    }
    updateFavorites() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = Object.assign({}, vscode.workspace.getConfiguration().get(USSTree.persistenceSchema));
            if (settings.persistence) {
                settings.favorites = this.mFavorites.map((fav) => {
                    const correctedProfileName = "[" + fav.getProfileName() + "]: ";
                    return (fav.fullPath.startsWith(correctedProfileName) ? fav.fullPath : correctedProfileName + fav.fullPath) + "{" +
                        contextually.getBaseContext(fav) + "}";
                });
                yield vscode.workspace.getConfiguration().update(USSTree.persistenceSchema, settings, vscode.ConfigurationTarget.Global);
            }
        });
    }
    /**
     * Searches the loaded USS tree for items whose name contains a search string
     *
     */
    searchInLoadedItems() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.log) {
                this.log.debug(localize(2, null));
            }
            const loadedNodes = [];
            const sessions = yield this.getChildren();
            // Add all data sets loaded in the tree to an array
            for (const session of sessions) {
                if (!session.contextValue.includes(globals.FAVORITE_CONTEXT)) {
                    const nodes = yield session.getChildren();
                    const checkForChildren = (nodeToCheck) => __awaiter(this, void 0, void 0, function* () {
                        const children = nodeToCheck.children;
                        if (children.length !== 0) {
                            for (const child of children) {
                                yield checkForChildren(child);
                            }
                        }
                        loadedNodes.push(nodeToCheck);
                    });
                    if (nodes) {
                        for (const node of nodes) {
                            yield checkForChildren(node);
                        }
                    }
                }
            }
            return loadedNodes;
        });
    }
    /**
     * Prompts the user for a path, and populates the [TreeView]{@link vscode.TreeView} based on the path
     *
     * @param {IZoweUSSTreeNode} node - The session node
     * @returns {Promise<void>}
     */
    filterPrompt(node) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.log) {
                this.log.debug(localize(3, null));
            }
            let sessionNode = node.getSessionNode();
            let remotepath;
            yield this.checkCurrentProfile(node);
            if (Profiles_1.Profiles.getInstance().validProfile === Profiles_1.ValidProfileEnum.VALID) {
                if (contextually.isSessionNotFav(node)) {
                    if (this.mHistory.getHistory().length > 0) {
                        const createPick = new utils_1.FilterDescriptor(USSTree.defaultDialogText);
                        const items = this.mHistory.getHistory().map((element) => new utils_1.FilterItem(element));
                        if (globals.ISTHEIA) {
                            const options1 = {
                                placeHolder: localize(4, null)
                            };
                            // get user selection
                            const choice = (yield vscode.window.showQuickPick([createPick, ...items], options1));
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(5, null));
                                return;
                            }
                            remotepath = choice === createPick ? "" : choice.label;
                        }
                        else {
                            const quickpick = vscode.window.createQuickPick();
                            quickpick.placeholder = localize(6, null);
                            quickpick.items = [createPick, ...items];
                            quickpick.ignoreFocusOut = true;
                            quickpick.show();
                            const choice = yield utils_1.resolveQuickPickHelper(quickpick);
                            quickpick.hide();
                            if (!choice) {
                                vscode.window.showInformationMessage(localize(7, null));
                                return;
                            }
                            if (choice instanceof utils_1.FilterDescriptor) {
                                if (quickpick.value) {
                                    remotepath = quickpick.value;
                                }
                            }
                            else {
                                remotepath = choice.label;
                            }
                        }
                    }
                    if (!remotepath) {
                        // manually entering a search - switch to an input box
                        const options = {
                            prompt: localize(8, null),
                            value: sessionNode.fullPath
                        };
                        // get user input
                        remotepath = yield vscode.window.showInputBox(options);
                        if (!remotepath || remotepath.length === 0) {
                            vscode.window.showInformationMessage(localize(9, null));
                            return;
                        }
                    }
                }
                else {
                    // executing search from saved search in favorites
                    remotepath = node.label.trim().substring(node.label.trim().indexOf(":") + 2);
                    const session = node.label.trim().substring(node.label.trim().indexOf("[") + 1, node.label.trim().indexOf("]"));
                    yield this.addSession(session);
                    const faveNode = node;
                    sessionNode = this.mSessionNodes.find((tempNode) => tempNode.getProfileName() === session);
                    if ((!sessionNode.getSession().ISession.user) || (!sessionNode.getSession().ISession.password)) {
                        sessionNode.getSession().ISession.user = faveNode.getSession().ISession.user;
                        sessionNode.getSession().ISession.password = faveNode.getSession().ISession.password;
                        sessionNode.getSession().ISession.base64EncodedAuth = faveNode.getSession().ISession.base64EncodedAuth;
                    }
                }
                // Sanitization: Replace multiple preceding forward slashes with just one forward slash
                const sanitizedPath = remotepath.replace(/\/\/+/, "/");
                sessionNode.tooltip = sessionNode.fullPath = sanitizedPath;
                sessionNode.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
                const icon = icons_1.getIconByNode(sessionNode);
                if (icon) {
                    sessionNode.iconPath = icon.path;
                }
                // update the treeview with the new path
                sessionNode.label = `${sessionNode.getProfileName()} [${sanitizedPath}]`;
                sessionNode.dirty = true;
                this.addHistory(sanitizedPath);
            }
        });
    }
    initialize(log) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log = log;
            this.log.debug(localize(10, null));
            const favoriteSearchPattern = /^\[.+\]\:\s.*\{ussSession\}$/;
            const directorySearchPattern = /^\[.+\]\:\s.*\{directory\}$/;
            const lines = this.mHistory.readFavorites();
            lines.forEach((line) => {
                const profileName = line.substring(1, line.lastIndexOf("]"));
                const nodeName = (line.substring(line.indexOf(":") + 1, line.indexOf("{"))).trim();
                const sesName = line.substring(1, line.lastIndexOf("]")).trim();
                try {
                    const profile = Profiles_1.Profiles.getInstance().loadNamedProfile(sesName);
                    const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(profile).getSession();
                    let node;
                    if (directorySearchPattern.test(line)) {
                        node = new ZoweUSSNode_1.ZoweUSSNode(nodeName, vscode.TreeItemCollapsibleState.Collapsed, this.mFavoriteSession, session, "", false, profileName);
                    }
                    else if (favoriteSearchPattern.test(line)) {
                        const label = "[" + sesName + "]: " + nodeName;
                        node = new ZoweUSSNode_1.ZoweUSSNode(label, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, null, false, profileName);
                        node.contextValue = globals.USS_SESSION_CONTEXT;
                        node.fullPath = nodeName;
                        node.label = node.tooltip = label;
                        // add a command to execute the search
                        node.command = { command: "zowe.uss.fullPath", title: "", arguments: [node] };
                    }
                    else {
                        node = new ZoweUSSNode_1.ZoweUSSNode(nodeName, vscode.TreeItemCollapsibleState.None, this.mFavoriteSession, session, "", false, profileName);
                        node.command = { command: "zowe.uss.ZoweUSSNode.open",
                            title: localize(11, null), arguments: [node] };
                    }
                    node.contextValue = contextually.asFavorite(node);
                    const icon = icons_1.getIconByNode(node);
                    if (icon) {
                        node.iconPath = icon.path;
                    }
                    this.mFavorites.push(node);
                }
                catch (e) {
                    const errMessage = localize(12, null) + profileName +
                        localize(13, null) +
                        localize(14, null) +
                        utils_2.getAppName(globals.ISTHEIA) + localize(15, null);
                    utils_1.errorHandling(e, null, errMessage);
                    return;
                }
            });
        });
    }
    addRecall(criteria) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mHistory.addRecall(criteria);
            this.refresh();
        });
    }
    getRecall() {
        return this.mHistory.getRecall();
    }
    removeRecall(name) {
        this.mHistory.removeRecall(name);
    }
    /**
     * Opens a USS item & reveals it in the tree
     *
     */
    openItemFromPath(itemPath, sessionNode) {
        return __awaiter(this, void 0, void 0, function* () {
            // USS file was selected
            const nodePath = itemPath.substring(itemPath.indexOf("/") + 1).trim().split("/");
            const selectedNodeName = nodePath[nodePath.length - 1];
            // Update the tree filter & expand the tree
            sessionNode.collapsibleState = vscode.TreeItemCollapsibleState.Expanded;
            sessionNode.tooltip = sessionNode.fullPath = `/${nodePath.slice(0, nodePath.length - 1).join("/")}`;
            sessionNode.label = `${sessionNode.getProfileName()} [/${nodePath.join("/")}]`;
            sessionNode.dirty = true;
            this.addHistory(`[${sessionNode.getProfileName()}]: /${nodePath.join("/")}`);
            yield sessionNode.getChildren();
            // Reveal the searched item in the tree
            const selectedNode = sessionNode.children.find((elt) => elt.label === selectedNodeName);
            if (selectedNode) {
                selectedNode.openUSS(false, true, this);
            }
            else {
                vscode.window.showInformationMessage(localize(16, null));
                this.removeRecall(`[${sessionNode.getProfileName()}]: ${itemPath}`);
            }
        });
    }
    checkCurrentProfile(node) {
        return __awaiter(this, void 0, void 0, function* () {
            const profile = node.getProfile();
            yield Profiles_1.Profiles.getInstance().checkCurrentProfile(profile);
            yield this.refresh();
        });
    }
    /**
     * Adds a single session to the USS tree
     *
     */
    addSingleSession(profile) {
        return __awaiter(this, void 0, void 0, function* () {
            if (profile) {
                // If session is already added, do nothing
                if (this.mSessionNodes.find((tempNode) => tempNode.label.trim() === profile.name)) {
                    return;
                }
                // Uses loaded profile to create a session with the USS API
                const session = ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(profile).getSession();
                // Creates ZoweNode to track new session and pushes it to mSessionNodes
                const node = new ZoweUSSNode_1.ZoweUSSNode(profile.name, vscode.TreeItemCollapsibleState.Collapsed, null, session, "", false, profile.name);
                node.contextValue = globals.USS_SESSION_CONTEXT;
                const icon = icons_1.getIconByNode(node);
                if (icon) {
                    node.iconPath = icon.path;
                }
                node.dirty = true;
                this.mSessionNodes.push(node);
                this.mHistory.addSession(profile.name);
            }
        });
    }
}
exports.USSTree = USSTree;
USSTree.defaultDialogText = "\uFF0B " + localize(17, null);
USSTree.persistenceSchema = "Zowe-USS-Persistent";

//# sourceMappingURL=USSTree.js.map
