"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const globals = require("../globals");
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");
const moment = require("moment");
const utils_1 = require("../utils");
const ZoweTreeNode_1 = require("../abstract/ZoweTreeNode");
const index_1 = require("../generators/icons/index");
const utils_2 = require("../uss/utils");
const Profiles_1 = require("../Profiles");
const ZoweExplorerApiRegister_1 = require("../api/ZoweExplorerApiRegister");
const contextually = require("../shared/context");
const nls = require("vscode-nls");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
/**
 * A type of TreeItem used to represent sessions and USS directories and files
 *
 * @export
 * @class ZoweUSSNode
 * @extends {vscode.TreeItem}
 */
class ZoweUSSNode extends ZoweTreeNode_1.ZoweTreeNode {
    /**
     * Creates an instance of ZoweUSSNode
     *
     * @param {string} label - Displayed in the [TreeView]
     * @param {vscode.TreeItemCollapsibleState} collapsibleState - file/directory
     * @param {IZoweUSSTreeNode} mParent - The parent node
     * @param {Session} session
     * @param {String} parentPath - The file path of the parent on the server
     * @param {boolean} binary - Indictaes if this is a text or binary file
     * @param {String} mProfileName - Profile to which the node belongs to
     */
    constructor(label, collapsibleState, mParent, session, parentPath, binary = false, mProfileName, etag = "", profile) {
        super(label, collapsibleState, mParent, session, profile);
        this.parentPath = parentPath;
        this.mProfileName = mProfileName;
        this.etag = etag;
        this.fullPath = "";
        this.dirty = true;
        this.children = [];
        this.binaryFiles = {};
        this.binary = false;
        this.profileName = "";
        this.shortLabel = "";
        this.downloadedTime = null;
        this.downloadedInternal = false;
        this.binary = binary;
        if (collapsibleState !== vscode.TreeItemCollapsibleState.None) {
            this.contextValue = globals.USS_DIR_CONTEXT;
        }
        else if (binary) {
            this.contextValue = globals.DS_BINARY_FILE_CONTEXT;
        }
        else {
            this.contextValue = globals.DS_TEXT_FILE_CONTEXT;
        }
        if (this.parentPath) {
            this.fullPath = this.tooltip = this.parentPath + "/" + label;
            if (parentPath === "/") {
                // Keep fullPath of root level nodes preceded by a single slash
                this.fullPath = this.tooltip = "/" + label;
            }
        }
        if (mParent && contextually.isFavoriteContext(mParent)) {
            this.profileName = "[" + mProfileName + "]: ";
            this.fullPath = label.trim();
            // File or directory name only (no parent path)
            this.shortLabel = this.fullPath.split("/", this.fullPath.length).pop();
            // Display name for favorited file or directory in tree view
            this.label = this.profileName + this.shortLabel;
            this.tooltip = this.profileName + this.fullPath;
        }
        // TODO: this should not be necessary of each node gets initialized with the profile reference.
        if (mProfileName) {
            this.setProfile(Profiles_1.Profiles.getInstance().loadNamedProfile(mProfileName));
        }
        else if (mParent && mParent.mProfileName) {
            this.mProfileName = mParent.mProfileName;
            this.setProfile(Profiles_1.Profiles.getInstance().loadNamedProfile(mParent.mProfileName));
        }
        this.etag = etag ? etag : "";
        const icon = index_1.getIconByNode(this);
        if (icon) {
            this.iconPath = icon.path;
        }
    }
    /**
     * Implements access tto profile name
     * for {IZoweUSSTreeNode}.
     *
     * @returns {string}
     */
    getProfileName() {
        return this.returnmProfileName();
    }
    getSessionNode() {
        return this.session ? this : this.getParent().getSessionNode();
    }
    /**
     * Retrieves child nodes of this IZoweTreeNode
     *
     * @returns {Promise<IZoweUSSTreeNode[]>}
     */
    getChildren() {
        return __awaiter(this, void 0, void 0, function* () {
            if ((!this.fullPath && contextually.isSession(this)) || contextually.isDocument(this)) {
                return [];
            }
            if (!this.dirty) {
                if (this.collapsibleState === vscode.TreeItemCollapsibleState.Collapsed) {
                    this.children = [];
                }
                return this.children;
            }
            if (!this.label) {
                vscode.window.showErrorMessage(localize(0, null));
                throw Error("Invalid node");
            }
            // Gets the directories from the fullPath and displays any thrown errors
            const responses = [];
            try {
                responses.push(yield vscode.window.withProgress({
                    location: vscode.ProgressLocation.Notification,
                    title: localize(1, null)
                }, () => {
                    return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(this.getProfile()).fileList(this.fullPath);
                }));
            }
            catch (err) {
                utils_1.errorHandling(err, this.label, localize(2, null) + `uss-file-list`);
            }
            // push nodes to an object with property names to avoid duplicates
            const elementChildren = {};
            responses.forEach((response) => {
                // Throws reject if the Zowe command does not throw an error but does not succeed
                if (!response.success) {
                    throw Error(localize(3, null));
                }
                // Loops through all the returned file references members and creates nodes for them
                for (const item of response.apiResponse.items) {
                    const existing = this.children.find((element) => element.label.trim() === item.name);
                    if (existing) {
                        elementChildren[existing.label] = existing;
                    }
                    else if (item.name !== "." && item.name !== "..") {
                        // Creates a ZoweUSSNode for a directory
                        if (item.mode.startsWith("d")) {
                            const temp = new ZoweUSSNode(item.name, vscode.TreeItemCollapsibleState.Collapsed, this, null, this.fullPath, false, item.mProfileName);
                            elementChildren[temp.label] = temp;
                        }
                        else {
                            // Creates a ZoweUSSNode for a file
                            let temp;
                            if (this.getSessionNode().binaryFiles.hasOwnProperty(this.fullPath + "/" + item.name)) {
                                temp = new ZoweUSSNode(item.name, vscode.TreeItemCollapsibleState.None, this, null, this.fullPath, true, item.mProfileName);
                            }
                            else {
                                temp = new ZoweUSSNode(item.name, vscode.TreeItemCollapsibleState.None, this, null, this.fullPath, false, item.mProfileName);
                            }
                            temp.command = {
                                command: "zowe.uss.ZoweUSSNode.open",
                                title: localize(4, null), arguments: [temp]
                            };
                            elementChildren[temp.label] = temp;
                        }
                    }
                }
            });
            if (contextually.isSession(this)) {
                this.dirty = false;
            }
            return this.children = Object.keys(elementChildren).sort().map((labels) => elementChildren[labels]);
        });
    }
    setBinary(binary) {
        this.binary = binary;
        if (this.binary) {
            this.contextValue = globals.DS_BINARY_FILE_CONTEXT;
            this.getSessionNode().binaryFiles[this.fullPath] = true;
        }
        else {
            this.contextValue = globals.DS_TEXT_FILE_CONTEXT;
            delete this.getSessionNode().binaryFiles[this.fullPath];
        }
        if (this.getParent() && contextually.isFavoriteContext(this.getParent())) {
            this.binary ? this.contextValue = globals.DS_BINARY_FILE_CONTEXT + globals.FAV_SUFFIX :
                this.contextValue = globals.DS_TEXT_FILE_CONTEXT + globals.FAV_SUFFIX;
        }
        const icon = index_1.getIconByNode(this);
        if (icon) {
            this.setIcon(icon.path);
        }
        this.dirty = true;
    }
    /**
     * Helper getter to check dirtiness of node inside opened editor tabs, can be more accurate than saved value
     *
     * @returns {boolean}
     */
    get isDirtyInEditor() {
        const openedTextDocuments = vscode.workspace.textDocuments;
        const currentFilePath = this.getUSSDocumentFilePath();
        for (const document of openedTextDocuments) {
            if (document.fileName === currentFilePath) {
                return document.isDirty;
            }
        }
        return false;
    }
    get openedDocumentInstance() {
        const openedTextDocuments = vscode.workspace.textDocuments;
        const currentFilePath = this.getUSSDocumentFilePath();
        for (const document of openedTextDocuments) {
            if (document.fileName === currentFilePath) {
                return document;
            }
        }
        return null;
    }
    /**
     * helper method to change the node names in one go
     * @param oldReference string
     * @param revision string
     */
    rename(newFullPath) {
        return __awaiter(this, void 0, void 0, function* () {
            this.fullPath = newFullPath;
            this.shortLabel = newFullPath.split("/").pop();
            this.label = this.shortLabel;
            this.tooltip = utils_2.injectAdditionalDataToTooltip(this, newFullPath);
            if (this.isFolder) {
                yield vscode.commands.executeCommand("zowe.uss.refreshAll");
            }
            else {
                yield vscode.commands.executeCommand("zowe.uss.refreshUSSInTree", this);
                yield vscode.commands.executeCommand("zowe.uss.ZoweUSSNode.open", this);
            }
        });
    }
    /**
     * Helper method which sets an icon of node and initiates reloading of tree
     * @param iconPath
     */
    setIcon(iconPath) {
        this.iconPath = iconPath;
        vscode.commands.executeCommand("zowe.uss.refreshUSSInTree", this);
    }
    deleteUSSNode(ussFileProvider, filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            const quickPickOptions = {
                placeHolder: localize(5, null) + this.label,
                ignoreFocusOut: true,
                canPickMany: false
            };
            if ((yield vscode.window.showQuickPick([localize(6, null),
                localize(7, null)], quickPickOptions)) !== localize(8, null)) {
                return;
            }
            try {
                yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(this.profile).delete(this.fullPath, contextually.isUssDirectory(this));
                this.getParent().dirty = true;
                try {
                    if (fs.existsSync(filePath)) {
                        fs.unlinkSync(filePath);
                    }
                    // tslint:disable-next-line: no-empty
                }
                catch (err) { }
            }
            catch (err) {
                vscode.window.showErrorMessage(localize(9, null) + err.message);
                throw (err);
            }
            // Remove node from the USS Favorites tree
            ussFileProvider.removeFavorite(this);
            ussFileProvider.removeRecall(`[${this.getProfileName()}]: ${this.parentPath}/${this.label}`);
            ussFileProvider.refresh();
        });
    }
    /**
     * Returns the [etag] for this node
     *
     * @returns {string}
     */
    getEtag() {
        return this.etag;
    }
    /**
     * Set the [etag] for this node
     *
     * @returns {void}
     */
    setEtag(etagValue) {
        this.etag = etagValue;
    }
    /**
     * Getter for downloaded property
     *
     * @returns boolean
     */
    get downloaded() {
        return this.downloadedInternal;
    }
    /**
     * Setter for downloaded property
     * @param value boolean
     */
    set downloaded(value) {
        this.downloadedInternal = value;
        if (value) {
            this.downloadedTime = moment().toISOString();
            this.tooltip = utils_2.injectAdditionalDataToTooltip(this, this.fullPath);
        }
        const icon = index_1.getIconByNode(this);
        if (icon) {
            this.setIcon(icon.path);
        }
    }
    /**
     * Getter for folder type
     */
    get isFolder() {
        return [globals.USS_DIR_CONTEXT, globals.USS_DIR_CONTEXT + globals.FAV_SUFFIX].indexOf(this.contextValue) > -1;
    }
    /**
     * Downloads and displays a file in a text editor view
     *
     * @param {IZoweTreeNode} node
     */
    openUSS(download = false, previewFile, ussFileProvider) {
        return __awaiter(this, void 0, void 0, function* () {
            yield ussFileProvider.checkCurrentProfile(this);
            if (Profiles_1.Profiles.getInstance().validProfile === Profiles_1.ValidProfileEnum.VALID) {
                try {
                    let label;
                    switch (true) {
                        case (contextually.isFavoriteContext(this.getParent())):
                            label = this.label.substring(this.label.indexOf(":") + 1).trim();
                            break;
                        // Handle file path for files in directories and favorited directories
                        case (contextually.isUssDirectory(this.getParent())):
                            label = this.fullPath;
                            break;
                        case (contextually.isUssSession(this.getParent())):
                            label = this.label;
                            break;
                        default:
                            vscode.window.showErrorMessage(localize(10, null));
                            throw Error(localize(11, null));
                    }
                    // if local copy exists, open that instead of pulling from mainframe
                    const documentFilePath = this.getUSSDocumentFilePath();
                    if (download || !fs.existsSync(documentFilePath)) {
                        const profile = this.getProfile();
                        const fullPath = this.fullPath;
                        const chooseBinary = this.binary ||
                            (yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(profile).isFileTagBinOrAscii(this.fullPath));
                        const response = yield vscode.window.withProgress({
                            location: vscode.ProgressLocation.Notification,
                            title: "Opening USS file..."
                        }, function downloadUSSFile() {
                            return ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(profile).getContents(fullPath, {
                                file: documentFilePath,
                                binary: chooseBinary,
                                returnEtag: true
                            });
                        });
                        this.downloaded = true;
                        this.setEtag(response.apiResponse.etag);
                    }
                    // Add document name to recently-opened files
                    ussFileProvider.addRecall(`[${this.getProfile().name}]: ${this.fullPath}`);
                    ussFileProvider.getTreeView().reveal(this, { select: true, focus: true, expand: false });
                    yield this.initializeFileOpening(documentFilePath, previewFile);
                }
                catch (err) {
                    yield utils_1.errorHandling(err, this.mProfileName, err.message);
                    throw (err);
                }
            }
        });
    }
    /**
     * Refreshes the passed node with current mainframe data
     *
     * @param {ZoweUSSNode} node - The node which represents the file
     */
    refreshUSS() {
        return __awaiter(this, void 0, void 0, function* () {
            let label;
            switch (true) {
                case (contextually.isUssDirectory(this.getParent())):
                    label = this.fullPath;
                    break;
                case (contextually.isUssSession(this.getParent())):
                    label = this.label;
                    break;
                default:
                    vscode.window.showErrorMessage(localize(12, null));
                    throw Error(localize(13, null));
            }
            try {
                const ussDocumentFilePath = this.getUSSDocumentFilePath();
                const isDirty = this.isDirtyInEditor;
                let wasSaved = false;
                if (isDirty) {
                    attachRecentSaveListener();
                    vscode.window.showTextDocument(this.openedDocumentInstance);
                    yield vscode.commands.executeCommand("workbench.action.closeActiveEditor");
                    wasSaved = getRecentSaveStatus();
                    disposeRecentSaveListener();
                }
                if ((isDirty && !this.isDirtyInEditor && !wasSaved) || !isDirty) {
                    const response = yield ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getUssApi(this.getProfile()).getContents(this.fullPath, {
                        file: ussDocumentFilePath,
                        binary: this.binary,
                        returnEtag: true
                    });
                    this.setEtag(response.apiResponse.etag);
                    this.downloaded = true;
                    if (isDirty) {
                        yield this.initializeFileOpening(ussDocumentFilePath, true);
                    }
                }
                else if (wasSaved) {
                    yield this.initializeFileOpening(ussDocumentFilePath, true);
                }
            }
            catch (err) {
                if (err.message.includes(localize(14, null))) {
                    vscode.window.showInformationMessage(localize(15, null) + label +
                        localize(16, null));
                }
                else {
                    yield utils_1.errorHandling(err, this.mProfileName, err.message);
                }
            }
        });
    }
    initializeFileOpening(documentPath, previewFile) {
        return __awaiter(this, void 0, void 0, function* () {
            let document;
            let openingTextFailed = false;
            if (!this.binary) {
                try {
                    document = yield vscode.workspace.openTextDocument(documentPath);
                }
                catch (err) {
                    openingTextFailed = true;
                }
                if (openingTextFailed) {
                    const yesResponse = localize(17, null);
                    const noResponse = localize(18, null);
                    const response = yield vscode.window.showErrorMessage(localize(19, null), ...[
                        yesResponse,
                        noResponse
                    ]);
                    if (response === yesResponse.toString()) {
                        yield vscode.commands.executeCommand("zowe.uss.binary", this);
                    }
                }
                else {
                    if (previewFile === true) {
                        yield vscode.window.showTextDocument(document);
                    }
                    else {
                        yield vscode.window.showTextDocument(document, { preview: false });
                    }
                }
            }
            else {
                const uriPath = vscode.Uri.file(documentPath);
                yield vscode.commands.executeCommand("vscode.open", uriPath);
            }
        });
    }
    /**
     * Returns the local file path for the ZoweUSSNode
     *
     */
    getUSSDocumentFilePath() {
        return path.join(globals.USS_DIR || "", "/" + this.getSessionNode().getProfileName() + "/", this.fullPath);
    }
    returnmProfileName() {
        return this.mProfileName;
    }
}
exports.ZoweUSSNode = ZoweUSSNode;
let wasSavedRecently = false;
let saveListener = null;
/**
 * Helper function which sets up listener for save wiping out the data after certain delay to prevent the fact of second save
 * @param wipeOutTime {number}
 */
function attachRecentSaveListener(wipeOutTime = 500) {
    if (saveListener) {
        saveListener.dispose();
    }
    saveListener = vscode.workspace.onDidSaveTextDocument(() => {
        wasSavedRecently = true;
        setTimeout(() => {
            wasSavedRecently = false;
        }, wipeOutTime);
    });
}
exports.attachRecentSaveListener = attachRecentSaveListener;
/**
 * Helper function which returns saved save flag
 *
 * @returns {boolean}
 */
function getRecentSaveStatus() {
    return wasSavedRecently;
}
exports.getRecentSaveStatus = getRecentSaveStatus;
/**
 * Helper function which disposes recent save listener
 */
function disposeRecentSaveListener() {
    if (saveListener) {
        saveListener.dispose();
    }
}
exports.disposeRecentSaveListener = disposeRecentSaveListener;

//# sourceMappingURL=ZoweUSSNode.js.map
